@@ -201,15 +201,16 @@ class actionAngleAdiabaticGrid(actionAngle):
     def _evaluate(self,*args,**kwargs):
         """
         NAME:
-           _evaluate
+           __call__ (_evaluate)
         PURPOSE:
            evaluate the actions (jr,lz,jz)
         INPUT:
            Either:
-              a) R,vR,vT,z,vz
-              b) Orbit instance: initial condition used if that's it, orbit(t)
-                 if there is a time given as well
-           scipy.integrate.quadrature keywords
+              a) R,vR,vT,z,vz[,phi]:
+                 1) floats: phase-space value for single object (phi is optional) (each can be a Quantity)
+                 2) numpy.ndarray: [N] phase-space values for N objects (each can be a Quantity)
+              b) Orbit instance: initial condition used if that's it, orbit(t) if there is a time given as well as the second argument
+           scipy.integrate.quadrature keywords (used when directly evaluating a point off the grid)
         OUTPUT:
            (jr,lz,jz)
         HISTORY:
@@ -242,7 +243,7 @@ class actionAngleAdiabaticGrid(actionAngle):
             indx= (R > self._Rmax)
             indx+= (R < self._Rmin)
             indx+= (Ez != 0.)*(numpy.log(Ez) > thisEzZmax)
-            indxc= True-indx
+            indxc= True^indx
             jz= numpy.empty(R.shape)
             if numpy.sum(indxc) > 0:
                 jz[indxc]= (self._jzInterp.ev(R[indxc],Ez[indxc]/thisEzZmax[indxc])\
@@ -283,7 +284,7 @@ class actionAngleAdiabaticGrid(actionAngle):
             indx+= (ERLz > self._Lzmax)
             indx+= ((ER-thisERRa)/(thisERRL-thisERRa) > 1.)
             indx+= ((ER-thisERRa)/(thisERRL-thisERRa) < 0.)
-            indxc= True-indx
+            indxc= True^indx
             jr= numpy.empty(R.shape)
             if numpy.sum(indxc) > 0:
                 jr[indxc]= (self._jrInterp.ev(ERLz[indxc],
