@@ -12,9 +12,11 @@
 #
 ###############################################################################
 import copy
+import warnings
 import numpy as nu
 from galpy.actionAngle_src.actionAngle import actionAngle
 from galpy.potential import IsochronePotential
+from galpy.util import galpyWarning
 _APY_LOADED= True
 try:
     from astropy import units
@@ -83,15 +85,15 @@ class actionAngleIsochrone(actionAngle):
     def _evaluate(self,*args,**kwargs):
         """
         NAME:
-           _evaluate
+           __call__ (_evaluate)
         PURPOSE:
            evaluate the actions (jr,lz,jz)
         INPUT:
            Either:
-              a) R,vR,vT,z,vz
-              b) Orbit instance: initial condition used if that's it, orbit(t)
-                 if there is a time given as well
-           scipy.integrate.quadrature keywords
+              a) R,vR,vT,z,vz[,phi]:
+                 1) floats: phase-space value for single object (phi is optional) (each can be a Quantity)
+                 2) numpy.ndarray: [N] phase-space values for N objects (each can be a Quantity)
+              b) Orbit instance: initial condition used if that's it, orbit(t) if there is a time given as well as the second argument
         OUTPUT:
            (jr,lz,jz)
         HISTORY:
@@ -133,15 +135,15 @@ class actionAngleIsochrone(actionAngle):
     def _actionsFreqs(self,*args,**kwargs):
         """
         NAME:
-           _actionsFreqs
+           actionsFreqs (_actionsFreqs)
         PURPOSE:
            evaluate the actions and frequencies (jr,lz,jz,Omegar,Omegaphi,Omegaz)
         INPUT:
            Either:
-              a) R,vR,vT,z,vz
-              b) Orbit instance: initial condition used if that's it, orbit(t)
-                 if there is a time given as well
-           scipy.integrate.quadrature keywords
+              a) R,vR,vT,z,vz[,phi]:
+                 1) floats: phase-space value for single object (phi is optional) (each can be a Quantity)
+                 2) numpy.ndarray: [N] phase-space values for N objects (each can be a Quantity)
+              b) Orbit instance: initial condition used if that's it, orbit(t) if there is a time given as well as the second argument
         OUTPUT:
             (jr,lz,jz,Omegar,Omegaphi,Omegaz)
         HISTORY:
@@ -189,16 +191,15 @@ class actionAngleIsochrone(actionAngle):
     def _actionsFreqsAngles(self,*args,**kwargs):
         """
         NAME:
-           _actionsFreqsAngles
+           actionsFreqsAngles (_actionsFreqsAngles)
         PURPOSE:
-           evaluate the actions, frequencies, and angles 
-           (jr,lz,jz,Omegar,Omegaphi,Omegaz,angler,anglephi,anglez)
+           evaluate the actions, frequencies, and angles (jr,lz,jz,Omegar,Omegaphi,Omegaz,angler,anglephi,anglez)
         INPUT:
            Either:
-              a) R,vR,vT,z,vz,phi (MUST HAVE PHI)
-              b) Orbit instance: initial condition used if that's it, orbit(t)
-                 if there is a time given as well
-           scipy.integrate.quadrature keywords
+              a) R,vR,vT,z,vz[,phi]:
+                 1) floats: phase-space value for single object (phi is optional) (each can be a Quantity)
+                 2) numpy.ndarray: [N] phase-space values for N objects (each can be a Quantity)
+              b) Orbit instance: initial condition used if that's it, orbit(t) if there is a time given as well as the second argument
         OUTPUT:
             (jr,lz,jz,Omegar,Omegaphi,Omegaz,angler,anglephi,anglez)
         HISTORY:
@@ -247,8 +248,11 @@ class actionAngleIsochrone(actionAngle):
             c= -self.amp/2./E-self.b
             e2= 1.-L2/self.amp/c*(1.+self.b/c)
             e= nu.sqrt(e2)
-            s= 1.+nu.sqrt(1.+(R**2.+z**2.)/self.b**2.)
-            coseta= 1/e*(1.-self.b/c*(s-2.))
+            if self.b == 0.:
+                coseta= 1/e*(1.-nu.sqrt(R**2.+z**2.)/c)
+            else:
+                s= 1.+nu.sqrt(1.+(R**2.+z**2.)/self.b**2.)
+                coseta= 1/e*(1.-self.b/c*(s-2.))
             pindx= (coseta > 1.)*(coseta < (1.+10.**-7.))
             coseta[pindx]= 1.
             pindx= (coseta < -1.)*(coseta > (-1.-10.**-7.))
@@ -289,9 +293,67 @@ class actionAngleIsochrone(actionAngle):
             Omega= phi-u
             anglephi= Omega
             anglephi[indx]-= anglez[indx]
-            anglephi[True-indx]+= anglez[True-indx]
+            anglephi[True^indx]+= anglez[True^indx]
             angler= angler % (2.*nu.pi)
             anglephi= anglephi % (2.*nu.pi)
             anglez= anglez % (2.*nu.pi)
             return (Jr,Jphi,Jz,Omegar,Omegaphi,Omegaz,angler,anglephi,anglez)
 
+    def _EccZmaxRperiRap(self,*args,**kwargs):
+        """
+        NAME:
+           _EccZmaxRperiRap
+        PURPOSE:
+           evaluate the eccentricity, maximum height above the plane, peri- and apocenter for an isochrone potential
+        INPUT:
+           Either:
+              a) R,vR,vT,z,vz[,phi]:
+                 1) floats: phase-space value for single object (phi is optional) (each can be a Quantity)
+                 2) numpy.ndarray: [N] phase-space values for N objects (each can be a Quantity)
+              b) Orbit instance: initial condition used if that's it, orbit(t) if there is a time given as well as the second argument
+        OUTPUT:
+           (e,zmax,rperi,rap)
+        HISTORY:
+           2017-12-22 - Written - Bovy (UofT)
+        """
+        if len(args) == 5: #R,vR.vT, z, vz pragma: no cover
+            R,vR,vT, z, vz= args
+        elif len(args) == 6: #R,vR.vT, z, vz, phi
+            R,vR,vT, z, vz, phi= args
+        else:
+            self._parse_eval_args(*args)
+            R= self._eval_R
+            vR= self._eval_vR
+            vT= self._eval_vT
+            z= self._eval_z
+            vz= self._eval_vz
+        if isinstance(R,float):
+            R= nu.array([R])
+            vR= nu.array([vR])
+            vT= nu.array([vT])
+            z= nu.array([z])
+            vz= nu.array([vz])
+        if self._c: #pragma: no cover
+            pass
+        else:
+            Lz= R*vT
+            Lx= -z*vT
+            Ly= z*vR-R*vz
+            L2= Lx*Lx+Ly*Ly+Lz*Lz
+            E= self._ip(R,z)+vR**2./2.+vT**2./2.+vz**2./2.
+            if self.b == 0:
+                warnings.warn("zmax for point-mass (b=0) isochrone potential is only approximate, because it assumes that zmax is attained at rap, which is not necessarily the case",galpyWarning)
+                a= -self.amp/2./E
+                me2= L2/self.amp/a
+                e= nu.sqrt(1.-me2)
+                rperi= a*(1.-e)
+                rap= a*(1.+e)
+            else:
+                smin= 0.5*((2.*E-self.amp/self.b)\
+                               +nu.sqrt((2.*E-self.amp/self.b)**2.
+                                   +2.*E*(4.*self.amp/self.b+L2/self.b**2.)))/E
+                smax= 2.-self.amp/E/self.b-smin
+                rperi= smin*nu.sqrt(1.-2./smin)*self.b
+                rap= smax*nu.sqrt(1.-2./smax)*self.b
+            return ((rap-rperi)/(rap+rperi),rap*nu.sqrt(1.-Lz**2./L2),
+                    rperi,rap)
