@@ -164,27 +164,31 @@ class diskdf(df):
                     return sc.real(self.eval(*vRvTRToEL(args[0]._orb.vxvv[1],
                                                         args[0]._orb.vxvv[2],
                                                         args[0]._orb.vxvv[0],
-                                                        self._beta)))
+                                                        self._beta,
+                                                        self._dftype)))
             else:
                 no= args[0](args[1])
                 return sc.real(self.eval(*vRvTRToEL(no._orb.vxvv[1],
                                                     no._orb.vxvv[2],
                                                     no._orb.vxvv[0],
-                                                    self._beta)))
+                                                    self._beta,
+                                                    self._dftype)))
         elif isinstance(args[0],list) \
                  and isinstance(args[0][0],Orbit):
             #Grab all of the vR, vT, and R
             vR= nu.array([o._orb.vxvv[1] for o in args[0]])
             vT= nu.array([o._orb.vxvv[2] for o in args[0]])
             R= nu.array([o._orb.vxvv[0] for o in args[0]])
-            return sc.real(self.eval(*vRvTRToEL(vR,vT,R,self._beta)))
+            return sc.real(self.eval(*vRvTRToEL(vR,vT,R,self._beta,
+                                                self._dftype)))
         elif isinstance(args[0],nu.ndarray) and \
                 not (hasattr(args[0],'isscalar') and args[0].isscalar):
             #Grab all of the vR, vT, and R
             vR= args[0][1]
             vT= args[0][2]
             R= args[0][0]
-            return sc.real(self.eval(*vRvTRToEL(vR,vT,R,self._beta)))
+            return sc.real(self.eval(*vRvTRToEL(vR,vT,R,self._beta,
+                                                self._dftype)))
         else:
             return sc.real(self.eval(*args))
 
@@ -192,7 +196,7 @@ class diskdf(df):
         """Call the DF, marginalizing over perpendicular velocity"""
         #Get l, vlos
         l= o.ll(obs=[1.,0.,0.],ro=1.)*_DEGTORAD
-        vlos= o.vlos(ro=1.,vo=1.,obs=[1.,0.,0.,0.,0.,0.])[0]
+        vlos= o.vlos(ro=1.,vo=1.,obs=[1.,0.,0.,0.,0.,0.])
         R= o.R(use_physical=False)
         phi= o.phi(use_physical=False)
         #Get local circular velocity, projected onto the los
@@ -237,7 +241,7 @@ class diskdf(df):
         """Call the DF, marginalizing over line-of-sight velocity"""
         #Get d, l, vperp
         l= o.ll(obs=[1.,0.,0.],ro=1.)*_DEGTORAD
-        vperp= o.vll(ro=1.,vo=1.,obs=[1.,0.,0.,0.,0.,0.])[0]
+        vperp= o.vll(ro=1.,vo=1.,obs=[1.,0.,0.,0.,0.,0.])
         R= o.R(use_physical=False)
         phi= o.phi(use_physical=False)
         #Get local circular velocity, projected onto the perpendicular 
@@ -281,91 +285,6 @@ class diskdf(df):
                                         vperp-vcircperp,vcirc,sigmaR1),
                                   **kwargs)[0]/math.fabs(sinalphaperp)*sigmaR1
         
-    def _dlnfdR(self,R,vR,vT):
-        #Calculate a bunch of stuff that we need
-        if self._beta == 0.:
-            E= vR**2./2.+vT**2./2.+sc.log(R)
-            xE= sc.exp(E-.5)
-            OE= xE**-1.
-            LCE= xE
-            dRedR= xE/R
-        else: #non-flat rotation curve
-            E= vR**2./2.+vT**2./2.+1./2./self._beta*R**(2.*self._beta)
-            xE= (2.*E/(1.+1./self._beta))**(1./2./self._beta)
-            OE= xE**(self._beta-1.)
-            LCE= xE**(self._beta+1.)
-            dRedR= xE/2./self._beta/E*R**(2.*self._beta-1.)
-        return self._dlnfdRe(R,vR,vT,E=E,xE=xE,OE=OE,LCE=LCE)*dRedR\
-            +self._dlnfdl(R,vR,vT,E=E,xE=xE,OE=OE)*vT
-            
-    def _dlnfdvR(self,R,vR,vT):
-        #Calculate a bunch of stuff that we need
-        if self._beta == 0.:
-            E= vR**2./2.+vT**2./2.+sc.log(R)
-            xE= sc.exp(E-.5)
-            OE= xE**-1.
-            LCE= xE
-            dRedvR= xE*vR
-        else: #non-flat rotation curve
-            E= vR**2./2.+vT**2./2.+1./2./self._beta*R**(2.*self._beta)
-            xE= (2.*E/(1.+1./self._beta))**(1./2./self._beta)
-            OE= xE**(self._beta-1.)
-            LCE= xE**(self._beta+1.)
-            dRedvR= xE/2./self._beta/E*vR
-        return self._dlnfdRe(R,vR,vT,E=E,xE=xE,OE=OE,LCE=LCE)*dRedvR
-            
-    def _dlnfdvT(self,R,vR,vT):
-        #Calculate a bunch of stuff that we need
-        if self._beta == 0.:
-            E= vR**2./2.+vT**2./2.+sc.log(R)
-            xE= sc.exp(E-.5)
-            OE= xE**-1.
-            LCE= xE
-            dRedvT= xE*vT
-        else: #non-flat rotation curve
-            E= vR**2./2.+vT**2./2.+1./2./self._beta*R**(2.*self._beta)
-            xE= (2.*E/(1.+1./self._beta))**(1./2./self._beta)
-            OE= xE**(self._beta-1.)
-            LCE= xE**(self._beta+1.)
-            dRedvT= xE/2./self._beta/E*vT
-        return self._dlnfdRe(R,vR,vT,E=E,xE=xE,OE=OE,LCE=LCE)*dRedvT\
-            +self._dlnfdl(R,vR,vT,E=E,xE=xE,OE=OE)*R
-           
-    def _dlnfdRe(self,R,vR,vT,E=None,xE=None,OE=None,LCE=None):
-        """d ln f(x,v) / d R_e"""
-        #Calculate a bunch of stuff that we need
-        if E is None or xE is None or OE is None or LCE is None:
-            if self._beta == 0.:
-                E= vR**2./2.+vT**2./2.+sc.log(R)
-                xE= sc.exp(E-.5)
-                OE= xE**-1.
-                LCE= xE
-            else: #non-flat rotation curve
-                E= vR**2./2.+vT**2./2.+1./2./self._beta*R**(2.*self._beta)
-                xE= (2.*E/(1.+1./self._beta))**(1./2./self._beta)
-                OE= xE**(self._beta-1.)
-                LCE= xE**(self._beta+1.)
-        L= R*vT
-        sigma2xE= self._surfaceSigmaProfile.sigma2(xE,log=False)
-        return (self._surfaceSigmaProfile.surfacemassDerivative(xE,log=True)\
-                 -(1.+OE*(L-LCE)/sigma2xE)*self._surfaceSigmaProfile.sigma2Derivative(xE,log=True)\
-                 +(L-LCE)/sigma2xE*(self._beta-1.)*xE**(self._beta-2.)\
-                 -OE*(self._beta+1.)/sigma2xE*xE**self._beta)
-
-    def _dlnfdl(self,R,vR,vT,E=None,xE=None,OE=None):
-        #Calculate a bunch of stuff that we need
-        if E is None or xE is None or OE is None:
-            if self._beta == 0.:
-                E= vR**2./2.+vT**2./2.+sc.log(R)
-                xE= sc.exp(E-.5)
-                OE= xE**-1.
-            else: #non-flat rotation curve
-                E= vR**2./2.+vT**2./2.+1./2./self._beta*R**(2.*self._beta)
-                xE= (2.*E/(1.+1./self._beta))**(1./2./self._beta)
-                OE= xE**(self._beta-1.)
-        sigma2xE= self._surfaceSigmaProfile.sigma2(xE,log=False)
-        return OE/sigma2xE
-
     @potential_physical_input
     @physical_conversion('velocity2',pop=True)        
     def targetSigma2(self,R,log=False):
@@ -990,7 +909,7 @@ class diskdf(df):
                                          epsrel=_EPSREL)[0]/sc.pi*norm/2.
 
     @potential_physical_input
-    @physical_conversion('frequency_kmskpc',pop=True)
+    @physical_conversion('frequency-kmskpc',pop=True)
     def oortA(self,R,romberg=False,nsigma=None,phi=0.):
         """
 
@@ -1040,7 +959,7 @@ class diskdf(df):
         return 0.5*(meanvphi/R-dmeanvRRdphi/R-dmeanvphidR)
 
     @potential_physical_input
-    @physical_conversion('frequency_kmskpc',pop=True)
+    @physical_conversion('frequency-kmskpc',pop=True)
     def oortB(self,R,romberg=False,nsigma=None,phi=0.):
         """
         NAME:
@@ -1089,7 +1008,7 @@ class diskdf(df):
         return 0.5*(-meanvphi/R+dmeanvRRdphi/R-dmeanvphidR)
 
     @potential_physical_input
-    @physical_conversion('frequency_kmskpc',pop=True)
+    @physical_conversion('frequency-kmskpc',pop=True)
     def oortC(self,R,romberg=False,nsigma=None,phi=0.):
         """
         NAME:
@@ -1136,7 +1055,7 @@ class diskdf(df):
         return 0.5*(-meanvr/R-dmeanvphiRdphi/R+dmeanvRdR)
 
     @potential_physical_input
-    @physical_conversion('frequency_kmskpc',pop=True)
+    @physical_conversion('frequency-kmskpc',pop=True)
     def oortK(self,R,romberg=False,nsigma=None,phi=0.):
         """
         NAME:
@@ -1924,6 +1843,91 @@ class dehnendf(diskdf):
                 dum= [o.turn_physical_on(ro=self._ro,vo=self._vo) for o in out]
         return out
 
+    def _dlnfdR(self,R,vR,vT):
+        #Calculate a bunch of stuff that we need
+        if self._beta == 0.:
+            E= vR**2./2.+vT**2./2.+sc.log(R)
+            xE= sc.exp(E-.5)
+            OE= xE**-1.
+            LCE= xE
+            dRedR= xE/R
+        else: #non-flat rotation curve
+            E= vR**2./2.+vT**2./2.+1./2./self._beta*R**(2.*self._beta)
+            xE= (2.*E/(1.+1./self._beta))**(1./2./self._beta)
+            OE= xE**(self._beta-1.)
+            LCE= xE**(self._beta+1.)
+            dRedR= xE/2./self._beta/E*R**(2.*self._beta-1.)
+        return self._dlnfdRe(R,vR,vT,E=E,xE=xE,OE=OE,LCE=LCE)*dRedR\
+            +self._dlnfdl(R,vR,vT,E=E,xE=xE,OE=OE)*vT
+            
+    def _dlnfdvR(self,R,vR,vT):
+        #Calculate a bunch of stuff that we need
+        if self._beta == 0.:
+            E= vR**2./2.+vT**2./2.+sc.log(R)
+            xE= sc.exp(E-.5)
+            OE= xE**-1.
+            LCE= xE
+            dRedvR= xE*vR
+        else: #non-flat rotation curve
+            E= vR**2./2.+vT**2./2.+1./2./self._beta*R**(2.*self._beta)
+            xE= (2.*E/(1.+1./self._beta))**(1./2./self._beta)
+            OE= xE**(self._beta-1.)
+            LCE= xE**(self._beta+1.)
+            dRedvR= xE/2./self._beta/E*vR
+        return self._dlnfdRe(R,vR,vT,E=E,xE=xE,OE=OE,LCE=LCE)*dRedvR
+            
+    def _dlnfdvT(self,R,vR,vT):
+        #Calculate a bunch of stuff that we need
+        if self._beta == 0.:
+            E= vR**2./2.+vT**2./2.+sc.log(R)
+            xE= sc.exp(E-.5)
+            OE= xE**-1.
+            LCE= xE
+            dRedvT= xE*vT
+        else: #non-flat rotation curve
+            E= vR**2./2.+vT**2./2.+1./2./self._beta*R**(2.*self._beta)
+            xE= (2.*E/(1.+1./self._beta))**(1./2./self._beta)
+            OE= xE**(self._beta-1.)
+            LCE= xE**(self._beta+1.)
+            dRedvT= xE/2./self._beta/E*vT
+        return self._dlnfdRe(R,vR,vT,E=E,xE=xE,OE=OE,LCE=LCE)*dRedvT\
+            +self._dlnfdl(R,vR,vT,E=E,xE=xE,OE=OE)*R
+           
+    def _dlnfdRe(self,R,vR,vT,E=None,xE=None,OE=None,LCE=None):
+        """d ln f(x,v) / d R_e"""
+        #Calculate a bunch of stuff that we need
+        if E is None or xE is None or OE is None or LCE is None:
+            if self._beta == 0.:
+                E= vR**2./2.+vT**2./2.+sc.log(R)
+                xE= sc.exp(E-.5)
+                OE= xE**-1.
+                LCE= xE
+            else: #non-flat rotation curve
+                E= vR**2./2.+vT**2./2.+1./2./self._beta*R**(2.*self._beta)
+                xE= (2.*E/(1.+1./self._beta))**(1./2./self._beta)
+                OE= xE**(self._beta-1.)
+                LCE= xE**(self._beta+1.)
+        L= R*vT
+        sigma2xE= self._surfaceSigmaProfile.sigma2(xE,log=False)
+        return (self._surfaceSigmaProfile.surfacemassDerivative(xE,log=True)\
+                 -(1.+OE*(L-LCE)/sigma2xE)*self._surfaceSigmaProfile.sigma2Derivative(xE,log=True)\
+                 +(L-LCE)/sigma2xE*(self._beta-1.)*xE**(self._beta-2.)\
+                 -OE*(self._beta+1.)/sigma2xE*xE**self._beta)
+
+    def _dlnfdl(self,R,vR,vT,E=None,xE=None,OE=None):
+        #Calculate a bunch of stuff that we need
+        if E is None or xE is None or OE is None:
+            if self._beta == 0.:
+                E= vR**2./2.+vT**2./2.+sc.log(R)
+                xE= sc.exp(E-.5)
+                OE= xE**-1.
+            else: #non-flat rotation curve
+                E= vR**2./2.+vT**2./2.+1./2./self._beta*R**(2.*self._beta)
+                xE= (2.*E/(1.+1./self._beta))**(1./2./self._beta)
+                OE= xE**(self._beta-1.)
+        sigma2xE= self._surfaceSigmaProfile.sigma2(xE,log=False)
+        return OE/sigma2xE
+
 class shudf(diskdf):
     """Shu's df (1969)"""
     def __init__(self,surfaceSigma=expSurfaceSigmaProfile,
@@ -2125,39 +2129,144 @@ class shudf(diskdf):
                 dum= [o.turn_physical_on(ro=self._ro,vo=self._vo) for o in out]
         return out
 
+    def _dlnfdR(self,R,vR,vT):
+        #Calculate a bunch of stuff that we need
+        E, L= vRvTRToEL(vR,vT,R,self._beta,self._dftype)
+        if self._beta == 0.:
+            xL= L
+            dRldR= vT
+            ECL= sc.log(xL)+0.5
+            dECLEdR= 0.
+        else: #non-flat rotation curve
+            xL= L**(1./(self._beta+1.))
+            dRldR= L**(1./(self._beta+1.))/R/(self._beta+1.)
+            ECL= 0.5*(1./self._beta+1.)*xL**(2.*self._beta)
+            dECLdRl= (1.+self._beta)*xL**(2.*self._beta-1)
+            dEdR= R**(2.*self._beta-1.)
+            dECLEdR= dECLdRl*dRldR-dEdR
+        sigma2xL= self._surfaceSigmaProfile.sigma2(xL,log=False)
+        return (self._surfaceSigmaProfile.surfacemassDerivative(xL,log=True)\
+                 -(1.+(ECL-E)/sigma2xL)*self._surfaceSigmaProfile.sigma2Derivative(xL,log=True))*dRldR\
+                 +dECLEdR/sigma2xL
+    
+    def _dlnfdvR(self,R,vR,vT):
+        #Calculate a bunch of stuff that we need
+        E, L= vRvTRToEL(vR,vT,R,self._beta,self._dftype)
+        if self._beta == 0.:
+            xL= L
+        else: #non-flat rotation curve
+            xL= L**(1./(self._beta+1.))
+        sigma2xL= self._surfaceSigmaProfile.sigma2(xL,log=False)
+        return -vR/sigma2xL
+    
+    def _dlnfdvT(self,R,vR,vT):
+        #Calculate a bunch of stuff that we need
+        E, L= vRvTRToEL(vR,vT,R,self._beta,self._dftype)
+        if self._beta == 0.:
+            xL= L
+            dRldvT= R
+            ECL= sc.log(xL)+0.5
+            dECLEdvT= 1./vT-vT
+        else: #non-flat rotation curve
+            xL= L**(1./(self._beta+1.))
+            dRldvT= L**(1./(self._beta+1.))/vT/(self._beta+1.)
+            ECL= 0.5*(1./self._beta+1.)*xL**(2.*self._beta)
+            dECLdRl= (1.+self._beta)*xL**(2.*self._beta-1)
+            dEdvT= vT
+            dECLEdvT= dECLdRl*dRldvT-dEdvT
+        sigma2xL= self._surfaceSigmaProfile.sigma2(xL,log=False)
+        return (self._surfaceSigmaProfile.surfacemassDerivative(xL,log=True)\
+                 -(1.+(ECL-E)/sigma2xL)*self._surfaceSigmaProfile.sigma2Derivative(xL,log=True))*dRldvT\
+                 +dECLEdvT/sigma2xL
+    
+class schwarzschilddf(shudf):
+    """Schwarzschild's df"""
+    def __init__(self,surfaceSigma=expSurfaceSigmaProfile,
+                 profileParams=(1./3.,1.0,0.2),
+                 correct=False,
+                 beta=0.,**kwargs):
+        """
+        NAME:
+           __init__
+        PURPOSE:
+           Initialize a Schwarzschild DF
+        INPUT:
+           surfaceSigma - instance or class name of the target 
+                      surface density and sigma_R profile 
+                      (default: both exponential)
+           profileParams - parameters of the surface and sigma_R profile:
+                      (xD,xS,Sro) where
+          
+                        xD - disk surface mass scalelength (can be Quantity)
+              
+                        xS - disk velocity dispersion scalelength (can be Quantity)
+                        
+                        Sro - disk velocity dispersion at Ro (can be Quantity)
+                        
+                        Directly given to the 'surfaceSigmaProfile class, so
+                        could be anything that class takes
+
+           beta - power-law index of the rotation curve
+
+           correct - if True, correct the DF
+
+           ro= distance from vantage point to GC (kpc; can be Quantity)
+
+           vo= circular velocity at ro (km/s; can be Quantity)
+
+           +DFcorrection kwargs (except for those already specified)
+
+        OUTPUT:
+
+           instance
+
+        HISTORY:
+
+            2017-09-17 - Written - Bovy (UofT)
+
+        """
+        # Schwarzschild == Shu w/ energy computed in epicycle approx.
+        # so all functions are the same as in Shu, only thing different is
+        # how E is computed
+        return diskdf.__init__(self,surfaceSigma=surfaceSigma,
+                               profileParams=profileParams,
+                               correct=correct,dftype='schwarzschild',
+                               beta=beta,**kwargs)
+    
+
 def _surfaceIntegrand(vR,vT,R,df,logSigmaR,logsigmaR2,sigmaR1,gamma):
     """Internal function that is the integrand for the surface mass integration"""
-    E,L= _vRpvTpRToEL(vR,vT,R,df._beta,sigmaR1,gamma)
+    E,L= _vRpvTpRToEL(vR,vT,R,df._beta,sigmaR1,gamma,df._dftype)
     return df.eval(E,L,logSigmaR,logsigmaR2)*2.*nu.pi/df._gamma #correct
 
 def _sigma2surfaceIntegrand(vR,vT,R,df,logSigmaR,logsigmaR2,sigmaR1,gamma):
     """Internal function that is the integrand for the sigma-squared times
     surface mass integration"""
-    E,L= _vRpvTpRToEL(vR,vT,R,df._beta,sigmaR1,gamma)
+    E,L= _vRpvTpRToEL(vR,vT,R,df._beta,sigmaR1,gamma,df._dftype)
     return vR**2.*df.eval(E,L,logSigmaR,logsigmaR2)*2.*nu.pi/df._gamma #correct
 
 def _vmomentsurfaceIntegrand(vR,vT,R,df,logSigmaR,logsigmaR2,sigmaR1,gamma,
                              n,m):
     """Internal function that is the integrand for the velocity moment times
     surface mass integration"""
-    E,L= _vRpvTpRToEL(vR,vT,R,df._beta,sigmaR1,gamma)
+    E,L= _vRpvTpRToEL(vR,vT,R,df._beta,sigmaR1,gamma,df._dftype)
     return vR**n*vT**m*df.eval(E,L,logSigmaR,logsigmaR2)*2.*nu.pi/df._gamma #correct
 
 def _vmomentderivsurfaceIntegrand(vR,vT,R,df,logSigmaR,logsigmaR2,sigmaR1,
                                   gamma,n,m,deriv):
     """Internal function that is the integrand for the derivative of velocity 
     moment times surface mass integration"""
-    E,L= _vRpvTpRToEL(vR,vT,R,df._beta,sigmaR1,gamma)
+    E,L= _vRpvTpRToEL(vR,vT,R,df._beta,sigmaR1,gamma,df._dftype)
     if deriv.lower() == 'r':
         return vR**n*vT**m*df.eval(E,L,logSigmaR,logsigmaR2)*2.*nu.pi/df._gamma*df._dlnfdR(R,vR*sigmaR1,vT*sigmaR1/gamma) #correct
     else:
         return 0.
 
-def _vRpvTpRToEL(vR,vT,R,beta,sigmaR1,gamma):
+def _vRpvTpRToEL(vR,vT,R,beta,sigmaR1,gamma,dftype='dehnen'):
     """Internal function that calculates E and L given velocities normalized by the velocity dispersion"""
     vR*= sigmaR1
     vT*= sigmaR1/gamma
-    return vRvTRToEL(vR,vT,R,beta)
+    return vRvTRToEL(vR,vT,R,beta,dftype)
 
 def _oned_intFunc(x,twodfunc,gfun,hfun,tol,args):
     """Internal function for bovy_dblquad"""
@@ -2394,7 +2503,7 @@ class DFcorrectionError(Exception):
     def __str__(self):
         return repr(self.value)
 
-def vRvTRToEL(vR,vT,R,beta):
+def vRvTRToEL(vR,vT,R,beta,dftype='dehnen'):
     """
     NAME:
        vRvTRToEL
@@ -2408,7 +2517,19 @@ def vRvTRToEL(vR,vT,R,beta):
     HISTORY:
        2010-03-10 - Written - Bovy (NYU)
     """
-    return (axipotential(R,beta)+0.5*vR**2.+0.5*vT**2.,vT*R)
+    if dftype == 'schwarzschild':
+        # Compute E in the epicycle approximation
+        gamma= sc.sqrt(2./(1.+beta))
+        L= R*vT
+        if beta == 0.:
+            xL= L
+        else: #non-flat rotation curve
+            xL= L**(1./(beta+1.))   
+        return (0.5*vR**2.+0.5*gamma**2.*(vT-R**beta)**2.
+                +xL**(2.*beta)/2.+axipotential(xL,beta=beta),
+                L)
+    else:
+        return (axipotential(R,beta)+0.5*vR**2.+0.5*vT**2.,vT*R)
 
 def axipotential(R,beta=0.):
     """
@@ -2522,10 +2643,10 @@ def _vtmaxEq(vT,R,diskdf):
 def _marginalizeVperpIntegrandSinAlphaLarge(vR,df,R,sinalpha,cotalpha,
                                             vlos,vcirc,sigma):
     return df(*vRvTRToEL(vR*sigma,cotalpha*vR*sigma+vlos/sinalpha+vcirc,
-                        R,df._beta))
+                        R,df._beta,df._dftype))
 
 def _marginalizeVperpIntegrandSinAlphaSmall(vT,df,R,cosalpha,tanalpha,
                                             vlos,vcirc,sigma):
     return df(*vRvTRToEL(tanalpha*vT*sigma-vlos/cosalpha,vT*sigma+vcirc,
-                        R,df._beta))
+                        R,df._beta,df._dftype))
 
