@@ -1,8 +1,10 @@
+import warnings
 import math as m
 import numpy as nu
 from scipy import integrate
 from galpy.potential_src.Potential import _evaluateRforces, _evaluatezforces,\
-    evaluatePotentials, evaluateDensities
+    evaluatePotentials, evaluateDensities, _check_c
+from galpy.util import galpyWarning
 import galpy.util.bovy_plot as plot
 import galpy.util.bovy_symplecticode as symplecticode
 from galpy.orbit_src.FullOrbit import _integrateFullOrbit
@@ -254,7 +256,7 @@ class RZOrbit(OrbitTop):
             kwargs.pop('use_physical')
         return out
 
-    def e(self,analytic=False,pot=None):
+    def e(self,analytic=False,pot=None,**kwargs):
         """
         NAME:
            e
@@ -269,11 +271,10 @@ class RZOrbit(OrbitTop):
            2010-09-15 - Written - Bovy (NYU)
         """
         if analytic:
-            self._setupaA(pot=pot,type='adiabatic')
-            (rperi,rap)= self._aA.calcRapRperi(self)
-            return (rap-rperi)/(rap+rperi)
+            self._setupaA(pot=pot,**kwargs)
+            return float(self._aA.EccZmaxRperiRap(self)[0])
         if not hasattr(self,'orbit'):
-            raise AttributeError("Integrate the orbit first")
+            raise AttributeError("Integrate the orbit first or use analytic=True for approximate eccentricity")
         if not hasattr(self,'rs'):
             self.rs= nu.sqrt(self.orbit[:,0]**2.+self.orbit[:,3]**2.)
         return (nu.amax(self.rs)-nu.amin(self.rs))/(nu.amax(self.rs)+nu.amin(self.rs))
@@ -294,11 +295,10 @@ class RZOrbit(OrbitTop):
            2010-09-20 - Written - Bovy (NYU)
         """
         if analytic:
-            self._setupaA(pot=pot,type='adiabatic')
-            (rperi,rap)= self._aA.calcRapRperi(self)
-            return rap
+            self._setupaA(pot=pot,**kwargs)
+            return float(self._aA.EccZmaxRperiRap(self)[3])
         if not hasattr(self,'orbit'):
-            raise AttributeError("Integrate the orbit first")
+            raise AttributeError("Integrate the orbit first or use analytic=True for approximate rap")
         if not hasattr(self,'rs'):
             self.rs= nu.sqrt(self.orbit[:,0]**2.+self.orbit[:,3]**2.)
         return nu.amax(self.rs)
@@ -319,11 +319,10 @@ class RZOrbit(OrbitTop):
            2010-09-20 - Written - Bovy (NYU)
         """
         if analytic:
-            self._setupaA(pot=pot,type='adiabatic')
-            (rperi,rap)= self._aA.calcRapRperi(self)
-            return rperi
+            self._setupaA(pot=pot,**kwargs)
+            return float(self._aA.EccZmaxRperiRap(self)[2])
         if not hasattr(self,'orbit'):
-            raise AttributeError("Integrate the orbit first")
+            raise AttributeError("Integrate the orbit first or use analytic=True for approximate rperi")
         if not hasattr(self,'rs'):
             self.rs= nu.sqrt(self.orbit[:,0]**2.+self.orbit[:,3]**2.)
         return nu.amin(self.rs)
@@ -342,11 +341,10 @@ class RZOrbit(OrbitTop):
            2010-09-20 - Written - Bovy (NYU)
         """
         if analytic:
-            self._setupaA(pot=pot,type='adiabatic')
-            zmax= self._aA.calczmax(self)
-            return zmax
+            self._setupaA(pot=pot,**kwargs)
+            return float(self._aA.EccZmaxRperiRap(self)[1])
         if not hasattr(self,'orbit'):
-            raise AttributeError("Integrate the orbit first")
+            raise AttributeError("Integrate the orbit first or use analytic=True for approximate zmax")
         return nu.amax(nu.fabs(self.orbit[:,3]))
 
     def plotEz(self,*args,**kwargs):
@@ -366,7 +364,7 @@ class RZOrbit(OrbitTop):
             kwargs['d2']= 'Eznorm'
         else:
             kwargs['d2']= 'Ez'
-        self.plot(*args,**kwargs)
+        return self.plot(*args,**kwargs)
         
     def plotER(self,*args,**kwargs):
         """
@@ -385,7 +383,7 @@ class RZOrbit(OrbitTop):
             kwargs['d2']= 'ERnorm'
         else:
             kwargs['d2']= 'ER'
-        self.plot(*args,**kwargs)
+        return self.plot(*args,**kwargs)
         
     def plotEzJz(self,*args,**kwargs):
         """
@@ -428,23 +426,29 @@ class RZOrbit(OrbitTop):
         if not 'ylabel' in kwargs:
             kwargs['ylabel']= r'$E_z/\sqrt{\rho}$'
         if d1 == 't':
-            plot.bovy_plot(nu.array(self.t),nu.array(self.EzJz)/self.EzJz[0],
-                           *args,**kwargs)
+            return plot.bovy_plot(nu.array(self.t),
+                                  nu.array(self.EzJz)/self.EzJz[0],
+                                  *args,**kwargs)
         elif d1 == 'z':
-            plot.bovy_plot(self.orbit[:,3],nu.array(self.EzJz)/self.EzJz[0],
-                           *args,**kwargs)
+            return plot.bovy_plot(self.orbit[:,3],
+                                  nu.array(self.EzJz)/self.EzJz[0],
+                                  *args,**kwargs)
         elif d1 == 'R':
-            plot.bovy_plot(self.orbit[:,0],nu.array(self.EzJz)/self.EzJz[0],
-                           *args,**kwargs)
+            return plot.bovy_plot(self.orbit[:,0],
+                                  nu.array(self.EzJz)/self.EzJz[0],
+                                  *args,**kwargs)
         elif d1 == 'vR':
-            plot.bovy_plot(self.orbit[:,1],nu.array(self.EzJz)/self.EzJz[0],
-                           *args,**kwargs)
+            return plot.bovy_plot(self.orbit[:,1],
+                                  nu.array(self.EzJz)/self.EzJz[0],
+                                  *args,**kwargs)
         elif d1 == 'vT':
-            plot.bovy_plot(self.orbit[:,2],nu.array(self.EzJz)/self.EzJz[0],
-                           *args,**kwargs)
+            return plot.bovy_plot(self.orbit[:,2],
+                                  nu.array(self.EzJz)/self.EzJz[0],
+                                  *args,**kwargs)
         elif d1 == 'vz':
-            plot.bovy_plot(self.orbit[:,4],nu.array(self.EzJz)/self.EzJz[0],
-                           *args,**kwargs)
+            return plot.bovy_plot(self.orbit[:,4],
+                                  nu.array(self.EzJz)/self.EzJz[0],
+                                  *args,**kwargs)
 
 def _integrateRZOrbit(vxvv,pot,t,method,dt):
     """
@@ -466,14 +470,12 @@ def _integrateRZOrbit(vxvv,pot,t,method,dt):
     """
     #First check that the potential has C
     if '_c' in method:
-        if isinstance(pot,list):
-            allHasC= nu.prod([p.hasC for p in pot])
-        else:
-            allHasC= pot.hasC
-        if not allHasC and ('leapfrog' in method or 'symplec' in method):
-            method= 'leapfrog'
-        elif not allHasC:
-            method= 'odeint'
+        if not _check_c(pot):
+            if ('leapfrog' in method or 'symplec' in method):
+                method= 'leapfrog'
+            else:
+                method= 'odeint'
+            warnings.warn("Cannot use C integration because some of the potentials are not implemented in C (using %s instead)" % (method), galpyWarning)
     if method.lower() == 'leapfrog' \
             or method.lower() == 'leapfrog_c' or method.lower() == 'rk4_c' \
             or method.lower() == 'rk6_c' or method.lower() == 'symplec4_c' \
