@@ -38,6 +38,7 @@ else:
 
 def _parse_pot(pot):
     """Parse the potential so it can be fed to C"""
+    from galpy.orbit_src.integrateFullOrbit import _parse_scf_pot
     #Figure out what's in pot
     if not isinstance(pot,list):
         pot= [pot]
@@ -46,14 +47,34 @@ def _parse_pot(pot):
     pot_args= []
     npot= len(pot)
     for p in pot:
-        if isinstance(p,potential_src.planarPotential.planarPotentialFromRZPotential) \
+        # Prepare for wrappers
+        if ((isinstance(p,\
+              potential_src.planarPotential.planarPotentialFromFullPotential) \
+          or isinstance(p,\
+              potential_src.planarPotential.planarPotentialFromRZPotential)) \
+          and isinstance(p._Pot,\
+              potential_src.WrapperPotential.parentWrapperPotential)) \
+        or isinstance(p,potential_src.WrapperPotential.parentWrapperPotential):
+            if not isinstance(p,\
+                     potential_src.WrapperPotential.parentWrapperPotential):
+                wrap_npot, wrap_pot_type, wrap_pot_args= \
+                    _parse_pot(potential.toPlanarPotential(p._Pot._pot))
+            else:
+                wrap_npot, wrap_pot_type, wrap_pot_args= _parse_pot(p._pot)
+        if (isinstance(p,potential_src.planarPotential.planarPotentialFromRZPotential) 
+            or isinstance(p,potential_src.planarPotential.planarPotentialFromFullPotential) ) \
                  and isinstance(p._Pot,potential.LogarithmicHaloPotential):
             pot_type.append(0)
-            pot_args.extend([p._Pot._amp,p._Pot._core2])
-        elif isinstance(p,potential.DehnenBarPotential):
+            if p._Pot.isNonAxi:
+                pot_args.extend([p._Pot._amp,p._Pot._core2,p._Pot._1m1overb2])
+            else:
+                pot_args.extend([p._Pot._amp,p._Pot._core2,2.]) # 1m1overb2 > 1: axi
+        elif isinstance(p,potential_src.planarPotential.planarPotentialFromFullPotential) \
+                 and isinstance(p._Pot,potential.DehnenBarPotential):
             pot_type.append(1)
-            pot_args.extend([p._amp,p._tform,p._tsteady,p._rb,p._af,p._omegab,
-                             p._barphi])
+            pot_args.extend([p._Pot._amp*p._Pot._af,p._Pot._tform,
+                             p._Pot._tsteady,p._Pot._rb,p._Pot._omegab,
+                             p._Pot._barphi])
         elif isinstance(p,potential.TransientLogSpiralPotential):
             pot_type.append(2)
             pot_args.extend([p._amp,p._A,p._to,p._sigma2,p._alpha,p._m,
@@ -81,12 +102,7 @@ def _parse_pot(pot):
             pot_args.extend([p._Pot._amp,p._Pot._a,p._Pot._b])
         elif isinstance(p,potential.LopsidedDiskPotential):
             pot_type.append(6)
-            if p._tform is None:
-                pot_args.extend([p._amp,float('nan'), float('nan'),
-                                 p._mphio,p._p,p._phib])
-            else:
-                pot_args.extend([p._amp,p._tform,p._tsteady,
-                                 p._mphio,p._p,p._phib])
+            pot_args.extend([p._amp,p._mphio,p._p,p._phib])
         elif isinstance(p,potential_src.planarPotential.planarPotentialFromRZPotential) \
                  and isinstance(p._Pot,potential.PowerSphericalPotential):
             pot_type.append(7)
@@ -185,14 +201,80 @@ def _parse_pot(pot):
             pot_args.extend([0.,0.,0.,0.,0.,0.]) 
         elif (isinstance(p,potential_src.planarPotential.planarPotentialFromFullPotential) or isinstance(p,potential_src.planarPotential.planarPotentialFromRZPotential)) \
                  and isinstance(p._Pot,potential.SCFPotential):
-            isNonAxi = p._Pot.isNonAxi
-            pot_type.append(24)
-            pot_args.extend([p._Pot._a, isNonAxi])
-            pot_args.extend(p._Pot._Acos.shape)
-            pot_args.extend(p._Pot._amp*p._Pot._Acos.flatten(order='C'))
-            if isNonAxi:
-                pot_args.extend(p._Pot._amp*p._Pot._Asin.flatten(order='C'))  
-            pot_args.extend([-1., 0, 0, 0, 0, 0, 0])   
+            pt,pa= _parse_scf_pot(p._Pot)
+            pot_type.append(pt)
+            pot_args.extend(pa)
+        elif isinstance(p,potential_src.planarPotential.planarPotentialFromFullPotential) \
+                 and isinstance(p._Pot,potential.SoftenedNeedleBarPotential):
+            pot_type.append(25)
+            pot_args.extend([p._Pot._amp,p._Pot._a,p._Pot._b,p._Pot._c2,
+                             p._Pot._pa,p._Pot._omegab])
+            pot_args.extend([0.,0.,0.,0.,0.,0.,0.]) # for caching
+        elif (isinstance(p,potential_src.planarPotential.planarPotentialFromFullPotential) or isinstance(p,potential_src.planarPotential.planarPotentialFromRZPotential)) \
+                and isinstance(p._Pot,potential.DiskSCFPotential):
+            # Need to pull this apart into: (a) SCF part, (b) constituent
+            # [Sigma_i,h_i] parts
+            # (a) SCF, multiply in any add'l amp
+            pt,pa= _parse_scf_pot(p._Pot._scf,extra_amp=p._Pot._amp)
+            pot_type.append(pt)
+            pot_args.extend(pa)
+            # (b) constituent [Sigma_i,h_i] parts
+            for Sigma,hz in zip(p._Pot._Sigma_dict,p._Pot._hz_dict):
+                npot+= 1
+                pot_type.append(26)
+                stype= Sigma.get('type','exp')
+                if stype == 'exp' \
+                        or (stype == 'exp' and 'Rhole' in Sigma):
+                    pot_args.extend([3,0,
+                                     4.*nu.pi*Sigma.get('amp',1.)*p._Pot._amp,
+                                     Sigma.get('h',1./3.)])
+                elif stype == 'expwhole' \
+                        or (stype == 'exp' and 'Rhole' in Sigma):
+                    pot_args.extend([4,1,
+                                     4.*nu.pi*Sigma.get('amp',1.)*p._Pot._amp,
+                                     Sigma.get('h',1./3.),
+                                     Sigma.get('Rhole',0.5)])
+                hztype= hz.get('type','exp')
+                if hztype == 'exp':
+                    pot_args.extend([0,hz.get('h',0.0375)])
+                elif hztype == 'sech2':
+                    pot_args.extend([1,hz.get('h',0.0375)])
+        elif isinstance(p, potential_src.planarPotential.planarPotentialFromFullPotential) \
+                and isinstance(p._Pot, potential.SpiralArmsPotential):
+            pot_type.append(27)
+            pot_args.extend([len(p._Pot._Cs), p._Pot._amp, p._Pot._N, p._Pot._sin_alpha,
+                             p._Pot._tan_alpha, p._Pot._r_ref, p._Pot._phi_ref, p._Pot._Rs, p._Pot._H, p._Pot._omega])
+            pot_args.extend(p._Pot._Cs)
+        elif isinstance(p,potential.CosmphiDiskPotential):
+            pot_type.append(28)
+            pot_args.extend([p._amp,p._mphio,p._p,p._mphib,p._m,
+                             p._rb,p._rbp,p._rb2p,p._r1p])
+        elif isinstance(p,potential.HenonHeilesPotential):
+            pot_type.append(29)
+            pot_args.extend([p._amp])
+        ############################## WRAPPERS ###############################
+        elif ((isinstance(p,potential_src.planarPotential.planarPotentialFromFullPotential) or isinstance(p,potential_src.planarPotential.planarPotentialFromRZPotential)) \
+              and isinstance(p._Pot,potential.DehnenSmoothWrapperPotential)) \
+              or isinstance(p,potential.DehnenSmoothWrapperPotential):
+            if not isinstance(p,potential.DehnenSmoothWrapperPotential):
+                p= p._Pot
+            pot_type.append(-1)
+            # wrap_pot_type, args, and npot obtained before this horrible if
+            pot_args.append(wrap_npot)
+            pot_type.extend(wrap_pot_type)
+            pot_args.extend(wrap_pot_args)
+            pot_args.extend([p._amp,p._tform,p._tsteady])
+        elif ((isinstance(p,potential_src.planarPotential.planarPotentialFromFullPotential) or isinstance(p,potential_src.planarPotential.planarPotentialFromRZPotential)) \
+          and isinstance(p._Pot,potential.SolidBodyRotationWrapperPotential)) \
+          or isinstance(p,potential.SolidBodyRotationWrapperPotential):
+            if not isinstance(p,potential.SolidBodyRotationWrapperPotential):
+                p= p._Pot
+            pot_type.append(-2)
+            # wrap_pot_type, args, and npot obtained before this horrible if
+            pot_args.append(wrap_npot)
+            pot_type.extend(wrap_pot_type)
+            pot_args.extend(wrap_pot_args)
+            pot_args.extend([p._amp,p._omega,p._pa])
     pot_type= nu.array(pot_type,dtype=nu.int32,order='C')
     pot_args= nu.array(pot_args,dtype=nu.float64,order='C')
     return (npot,pot_type,pot_args)
