@@ -4,7 +4,7 @@
 extern "C" {
 #endif
 #include <galpy_potentials.h>
-void parse_leapFuncArgs_Full(int, struct potentialArg *,int *,double *);
+void parse_leapFuncArgs_Full(int, struct potentialArg *,int **,double **);
 #ifdef __cplusplus
 }
 #endif
