@@ -8,7 +8,7 @@ from galpy.orbit_src.OrbitTop import OrbitTop
 from galpy.potential_src.planarPotential import _evaluateplanarRforces,\
     RZToplanarPotential, toPlanarPotential, _evaluateplanarphiforces,\
     _evaluateplanarPotentials
-from galpy.potential_src.Potential import Potential
+from galpy.potential_src.Potential import Potential, _check_c
 from galpy.util import galpyWarning
 #try:
 from galpy.orbit_src.integratePlanarOrbit import integratePlanarOrbit_c,\
@@ -72,7 +72,7 @@ class planarOrbitTop(OrbitTop):
             (rperi,rap)= self._aA.calcRapRperi(self)
             return (rap-rperi)/(rap+rperi)
         if not hasattr(self,'orbit'):
-            raise AttributeError("Integrate the orbit first")
+            raise AttributeError("Integrate the orbit first or use analytic=True for approximate eccentricity")
         if not hasattr(self,'rs'):
             self.rs= self.orbit[:,0]
         return (nu.amax(self.rs)-nu.amin(self.rs))/(nu.amax(self.rs)+nu.amin(self.rs))
@@ -472,7 +472,7 @@ class planarOrbit(planarOrbitTop):
             (rperi,rap)= self._aA.calcRapRperi(self)
             return (rap-rperi)/(rap+rperi)
         if not hasattr(self,'orbit'):
-            raise AttributeError("Integrate the orbit first")
+            raise AttributeError("Integrate the orbit first or use analytic=True for approximate eccentricity")
         if not hasattr(self,'rs'):
             self.rs= self.orbit[:,0]
         return (nu.amax(self.rs)-nu.amin(self.rs))/(nu.amax(self.rs)+nu.amin(self.rs))
@@ -497,14 +497,12 @@ def _integrateROrbit(vxvv,pot,t,method,dt):
     """
     #First check that the potential has C
     if '_c' in method:
-        if isinstance(pot,list):
-            allHasC= nu.prod([p.hasC for p in pot])
-        else:
-            allHasC= pot.hasC
-        if not allHasC and ('leapfrog' in method or 'symplec' in method):
-            method= 'leapfrog'
-        elif not allHasC:
-            method= 'odeint'
+        if not _check_c(pot):
+            if ('leapfrog' in method or 'symplec' in method):
+                method= 'leapfrog'
+            else:
+                method= 'odeint'
+            warnings.warn("Cannot use C integration because some of the potentials are not implemented in C (using %s instead)" % (method), galpyWarning)
     if method.lower() == 'leapfrog':
         #We hack this by putting in a dummy phi
         this_vxvv= nu.zeros(len(vxvv)+1)
@@ -578,14 +576,12 @@ def _integrateOrbit(vxvv,pot,t,method,dt):
     """
     #First check that the potential has C
     if '_c' in method:
-        if isinstance(pot,list):
-            allHasC= nu.prod([p.hasC for p in pot])
-        else:
-            allHasC= pot.hasC
-        if not allHasC and ('leapfrog' in method or 'symplec' in method):
-            method= 'leapfrog'
-        elif not allHasC:
-            method= 'odeint'
+        if not _check_c(pot):
+            if ('leapfrog' in method or 'symplec' in method):
+                method= 'leapfrog'
+            else:
+                method= 'odeint'
+            warnings.warn("Cannot use C integration because some of the potentials are not implemented in C (using %s instead)" % (method), galpyWarning)
     if method.lower() == 'leapfrog':
         #go to the rectangular frame
         this_vxvv= nu.array([vxvv[0]*nu.cos(vxvv[3]),
@@ -674,10 +670,7 @@ def _integrateOrbit_dxdv(vxvv,dxdv,pot,t,method,rectIn,rectOut):
     """
     #First check that the potential has C
     if '_c' in method:
-        if isinstance(pot,list):
-            allHasC= nu.prod([p.hasC and p.hasC_dxdv for p in pot])
-        else:
-            allHasC= pot.hasC and pot.hasC_dxdv
+        allHasC= _check_c(pot) and _check_c(pot,dxdv=True)
         if not allHasC and not 'leapfrog' in method and not 'symplec' in method:
             method= 'odeint'
             warnings.warn("Using odeint because not all used potential have adequate C implementations to integrate phase-space volumes",galpyWarning)
