@@ -1,6 +1,4 @@
 import warnings
-from galpy.util import galpyWarning
-warnings.warn("A major change in versions > 1.1 is that all galpy.potential functions and methods take the potential as the first argument; previously methods such as evaluatePotentials, evaluateDensities, etc. would be called with (R,z,Pot), now they are called as (Pot,R,z) for greater consistency across the codebase",galpyWarning)
 from galpy.potential_src import Potential
 from galpy.potential_src import planarPotential
 from galpy.potential_src import linearPotential
@@ -33,7 +31,14 @@ from galpy.potential_src import PlummerPotential
 from galpy.potential_src import PseudoIsothermalPotential
 from galpy.potential_src import KuzminDiskPotential
 from galpy.potential_src import TwoPowerTriaxialPotential
+from galpy.potential_src import FerrersPotential
 from galpy.potential_src import SCFPotential
+from galpy.potential_src import SoftenedNeedleBarPotential
+from galpy.potential_src import DiskSCFPotential
+from galpy.potential_src import SpiralArmsPotential
+from galpy.potential_src import HenonHeilesPotential
+from galpy.potential_src import DehnenSmoothWrapperPotential
+from galpy.potential_src import SolidBodyRotationWrapperPotential
 #
 # Functions
 #
@@ -125,7 +130,15 @@ TriaxialHernquistPotential= TwoPowerTriaxialPotential.TriaxialHernquistPotential
 TriaxialNFWPotential= TwoPowerTriaxialPotential.TriaxialNFWPotential
 TriaxialJaffePotential= TwoPowerTriaxialPotential.TriaxialJaffePotential
 TwoPowerTriaxialPotential= TwoPowerTriaxialPotential.TwoPowerTriaxialPotential
+FerrersPotential= FerrersPotential.FerrersPotential
 SCFPotential = SCFPotential.SCFPotential
+SoftenedNeedleBarPotential= SoftenedNeedleBarPotential.SoftenedNeedleBarPotential
+DiskSCFPotential = DiskSCFPotential.DiskSCFPotential
+SpiralArmsPotential = SpiralArmsPotential.SpiralArmsPotential
+HenonHeilesPotential= HenonHeilesPotential.HenonHeilesPotential
+#Wrappers
+DehnenSmoothWrapperPotential= DehnenSmoothWrapperPotential.DehnenSmoothWrapperPotential
+SolidBodyRotationWrapperPotential= SolidBodyRotationWrapperPotential.SolidBodyRotationWrapperPotential
 #Softenings
 PlummerSoftening= ForceSoftening.PlummerSoftening
 
