@@ -1,23 +1,23 @@
 ###############################################################################
 #   DehnenBarPotential: Dehnen (2000)'s bar potential
 ###############################################################################
-import math as m
+import numpy
 from galpy.util import bovy_conversion
-from galpy.potential_src.planarPotential import planarPotential, _APY_LOADED
+from galpy.potential_src.Potential import Potential, _APY_LOADED
 if _APY_LOADED:
     from astropy import units
-_degtorad= m.pi/180.
-class DehnenBarPotential(planarPotential):
-    """Class that implements the Dehnen bar potential (Dehnen 2000)
+_degtorad= numpy.pi/180.
+class DehnenBarPotential(Potential):
+    """Class that implements the Dehnen bar potential (`Dehnen 2000 <http://adsabs.harvard.edu/abs/2000AJ....119..800D>`__), generalized to 3D following `Monari et al. (2016) <http://adsabs.harvard.edu/abs/2016MNRAS.461.3835M>`__
 
     .. math::
 
-        \\Phi(R,\\phi) = A_b(t)\\,\\cos\\left(2\\,(\\phi-\\Omega_b\\,t)\\right))\\times \\begin{cases}
-        -(R_b/R)^3\\,, & \\text{for}\\ R \\geq R_b\\\\
-        (R/R_b)^3-2\\,, & \\text{for}\\ R\\leq R_b.
+        \\Phi(R,z,\\phi) = A_b(t)\\,\\cos\\left(2\\,(\\phi-\\Omega_b\\,t)\\right))\\,\\left(\\frac{R}{r}\\right)^2\\,\\times \\begin{cases}
+        -(R_b/r)^3\\,, & \\text{for}\\ r \\geq R_b\\\\
+        (r/R_b)^3-2\\,, & \\text{for}\\ r\\leq R_b.
         \\end{cases}
 
-    where
+    where :math:`r^2 = R^2+z^2` is the spherical radius and
 
     .. math::
 
@@ -45,6 +45,7 @@ class DehnenBarPotential(planarPotential):
        \\alpha = 3\\,\\frac{A_f}{v_0^2}\\,\\left(\\frac{R_b}{r_0}\\right)^3\,.
 
     """
+    normalize= property() # turn off normalize
     def __init__(self,amp=1.,omegab=None,rb=None,chi=0.8,
                  rolr=0.9,barphi=25.*_degtorad,
                  tform=-4.,tsteady=None,beta=0.,
@@ -96,8 +97,10 @@ class DehnenBarPotential(planarPotential):
 
            2010-11-24 - Started - Bovy (NYU)
 
+           2017-06-23 - Converted to 3D following Monari et al. (2016) - Bovy (UofT/CCA)
+
         """
-        planarPotential.__init__(self,amp=amp,ro=ro,vo=vo)
+        Potential.__init__(self,amp=amp,ro=ro,vo=vo)
         if _APY_LOADED and isinstance(barphi,units.Quantity):
             barphi= barphi.to(units.rad).value
         if _APY_LOADED and isinstance(rolr,units.Quantity):
@@ -111,13 +114,14 @@ class DehnenBarPotential(planarPotential):
             Af= Af.to(units.km**2/units.s**2).value/self._vo**2.
         self.hasC= True
         self.hasC_dxdv= True
+        self.isNonAxi= True
         self._barphi= barphi
         if omegab is None:
             self._rolr= rolr
             self._chi= chi
             self._beta= beta
             #Calculate omegab and rb
-            self._omegab= 1./((self._rolr**(1.-self._beta))/(1.+m.sqrt((1.+self._beta)/2.)))
+            self._omegab= 1./((self._rolr**(1.-self._beta))/(1.+numpy.sqrt((1.+self._beta)/2.)))
             self._rb= self._chi*self._omegab**(1./(self._beta-1.))
             self._alpha= alpha
             self._af= self._alpha/3./self._rb**3.
@@ -125,14 +129,14 @@ class DehnenBarPotential(planarPotential):
             self._omegab= omegab
             self._rb= rb
             self._af= Af
-        self._tb= 2.*m.pi/self._omegab
+        self._tb= 2.*numpy.pi/self._omegab
         self._tform= tform*self._tb
         if tsteady is None:
             self._tsteady= self._tform/2.
         else:
             self._tsteady= self._tform+tsteady*self._tb
 
-    def _evaluate(self,R,phi=0.,t=0.):
+    def _evaluate(self,R,z,phi=0.,t=0.):
         """
         NAME:
            _evaluate
@@ -140,10 +144,11 @@ class DehnenBarPotential(planarPotential):
            evaluate the potential at R,phi,t
         INPUT:
            R - Galactocentric cylindrical radius
+           z - vertical height
            phi - azimuth
            t - time
         OUTPUT:
-           Phi(R,phi,t)
+           Phi(R,z,phi,t)
         HISTORY:
            2010-11-24 - Started - Bovy (NYU)
         """
@@ -156,15 +161,18 @@ class DehnenBarPotential(planarPotential):
             smooth= (3./16.*xi**5.-5./8*xi**3.+15./16.*xi+.5)
         else: #bar is fully on
             smooth= 1.
-        if R <= self._rb:
-            return self._af*smooth*m.cos(2.*(phi-self._omegab*t-self._barphi))\
-                *((R/self._rb)**3.-2.)
+        r2= R**2.+z**2.
+        r= numpy.sqrt(r2)
+        if r <= self._rb:
+            return self._af*smooth*numpy.cos(2.*(phi-self._omegab*t-self._barphi))\
+                *((r/self._rb)**3.-2.)*R**2./r2
         else:
-            return -self._af*smooth*m.cos(2.*(phi-self._omegab*t-
-                                              self._barphi))\
-                                              *(self._rb/R)**3.
+            return -self._af*smooth*numpy.cos(2.*(phi-self._omegab*t-
+                                                  self._barphi))\
+                                                  *(self._rb/r)**3.\
+                                                  *R**2./r2
 
-    def _Rforce(self,R,phi=0.,t=0.):
+    def _Rforce(self,R,z,phi=0.,t=0.):
         """
         NAME:
            _Rforce
@@ -172,6 +180,7 @@ class DehnenBarPotential(planarPotential):
            evaluate the radial force for this potential
         INPUT:
            R - Galactocentric cylindrical radius
+           z - vertical height
            phi - azimuth
            t - time
         OUTPUT:
@@ -188,16 +197,17 @@ class DehnenBarPotential(planarPotential):
             smooth= (3./16.*xi**5.-5./8*xi**3.+15./16.*xi+.5)
         else: #bar is fully on
             smooth= 1.
-        if R <= self._rb:
-            return -3.*self._af*smooth*m.cos(2.*(phi-self._omegab*t
-                                              -self._barphi))\
-                                              *(R/self._rb)**3./R
+        r= numpy.sqrt(R**2.+z**2.)
+        if r <= self._rb:
+            return -self._af*smooth*numpy.cos(2.*(phi-self._omegab*t
+                                                  -self._barphi))\
+                    *((r/self._rb)**3.*R*(3.*R**2.+2.*z**2.)-4.*R*z**2.)/r**4.
         else:
-            return -3.*self._af*smooth*m.cos(2.*(phi-self._omegab*t-
-                                              self._barphi))\
-                                              *(self._rb/R)**3./R
+            return -self._af*smooth*numpy.cos(2.*(phi-self._omegab*t-
+                                                  self._barphi))\
+                    *(self._rb/r)**3.*R/r**4.*(3.*R**2.-2.*z**2.)
         
-    def _phiforce(self,R,phi=0.,t=0.):
+    def _phiforce(self,R,z,phi=0.,t=0.):
         """
         NAME:
            _phiforce
@@ -205,6 +215,7 @@ class DehnenBarPotential(planarPotential):
            evaluate the azimuthal force for this potential
         INPUT:
            R - Galactocentric cylindrical radius
+           z - vertical height
            phi - azimuth
            t - time
         OUTPUT:
@@ -221,16 +232,33 @@ class DehnenBarPotential(planarPotential):
             smooth= (3./16.*xi**5.-5./8*xi**3.+15./16.*xi+.5)
         else: #bar is fully on
             smooth= 1.
-        if R <= self._rb:
-            return 2.*self._af*smooth*m.sin(2.*(phi-self._omegab*t-
-                                                self._barphi))\
-                                                *((R/self._rb)**3.-2.)
+        r2= R**2.+z**2.
+        r= numpy.sqrt(r2)
+        if r <= self._rb:
+            return 2.*self._af*smooth*numpy.sin(2.*(phi-self._omegab*t-
+                                                    self._barphi))\
+                                                *((r/self._rb)**3.-2.)*R**2./r2
         else:
-            return -2.*self._af*smooth*m.sin(2.*(phi-self._omegab*t-
-                                                 self._barphi))\
-                                                 *(self._rb/R)**3.
+            return -2.*self._af*smooth*numpy.sin(2.*(phi-self._omegab*t-
+                                                     self._barphi))\
+                                                     *(self._rb/r)**3.*R**2./r2
 
-    def _R2deriv(self,R,phi=0.,t=0.):
+    def _zforce(self,R,z,phi=0.,t=0.):
+        """
+        NAME:
+           _zforce
+        PURPOSE:
+           evaluate the vertical  force for this potential
+        INPUT:
+           R - Galactocentric cylindrical radius
+           z - vertical height
+           phi - azimuth
+           t - time
+        OUTPUT:
+           the vertical force
+        HISTORY:
+           2017-06-23 - Written - Bovy (NYU)
+        """
         #Calculate relevant time
         if t < self._tform:
             smooth= 0.
@@ -240,16 +268,17 @@ class DehnenBarPotential(planarPotential):
             smooth= (3./16.*xi**5.-5./8*xi**3.+15./16.*xi+.5)
         else: #bar is fully on
             smooth= 1.
-        if R <= self._rb:
-            return 6.*self._af*smooth*m.cos(2.*(phi-self._omegab*t
-                                              -self._barphi))\
-                                              *(R/self._rb)**3./R**2.
+        r= numpy.sqrt(R**2.+z**2.)
+        if r <= self._rb:
+            return -self._af*smooth*numpy.cos(2.*(phi-self._omegab*t
+                                                  -self._barphi))\
+                   *((r/self._rb)**3.+4.)*R**2.*z/r**4.
         else:
-            return -12.*self._af*smooth*m.cos(2.*(phi-self._omegab*t-
+            return -5.*self._af*smooth*numpy.cos(2.*(phi-self._omegab*t-
                                                   self._barphi))\
-                                                  *(self._rb/R)**3./R**2.
+                    *(self._rb/r)**3.*R**2.*z/r**4.
         
-    def _phi2deriv(self,R,phi=0.,t=0.):
+    def _R2deriv(self,R,z,phi=0.,t=0.):
         #Calculate relevant time
         if t < self._tform:
             smooth= 0.
@@ -259,16 +288,20 @@ class DehnenBarPotential(planarPotential):
             smooth= (3./16.*xi**5.-5./8*xi**3.+15./16.*xi+.5)
         else: #bar is fully on
             smooth= 1.
-        if R <= self._rb:
-            return -4.*self._af*smooth*m.cos(2.*(phi-self._omegab*t-
-                                                 self._barphi))\
-                                                 *((R/self._rb)**3.-2.)
+        r= numpy.sqrt(R**2.+z**2.)
+        if r <= self._rb:
+            return self._af*smooth*numpy.cos(2.*(phi-self._omegab*t
+                                                 -self._barphi))\
+                *((r/self._rb)**3.*((9.*R**2.+2.*z**2.)/r**4.
+                                    -R**2./r**6.*(3.*R**2.+2.*z**2.))\
+                      +4.*z**2./r**6.*(4.*R**2.-r**2.))
         else:
-            return 4.*self._af*smooth*m.cos(2.*(phi-self._omegab*t-
+            return self._af*smooth*numpy.cos(2.*(phi-self._omegab*t-
                                                  self._barphi))\
-                                                 *(self._rb/R)**3.       
-
-    def _Rphideriv(self,R,phi=0.,t=0.):
+                *(self._rb/r)**3./r**6.*((r**2.-7.*R**2.)*(3.*R**2.-2.*z**2.)\
+                                             +6.*R**2.*r**2.)
+        
+    def _phi2deriv(self,R,z,phi=0.,t=0.):
         #Calculate relevant time
         if t < self._tform:
             smooth= 0.
@@ -278,15 +311,78 @@ class DehnenBarPotential(planarPotential):
             smooth= (3./16.*xi**5.-5./8*xi**3.+15./16.*xi+.5)
         else: #bar is fully on
             smooth= 1.
-        if R <= self._rb:
-            return -6.*self._af*smooth*m.sin(2.*(phi-self._omegab*t
-                                              -self._barphi))\
-                                              *(R/self._rb)**3./R
+        r= numpy.sqrt(R**2.+z**2.)
+        if r <= self._rb:
+            return -4.*self._af*smooth*numpy.cos(2.*(phi-self._omegab*t-
+                                                     self._barphi))\
+                                            *((r/self._rb)**3.-2.)*R**2./r**2.
         else:
-            return -6.*self._af*smooth*m.sin(2.*(phi-self._omegab*t-
-                                              self._barphi))\
-                                              *(self._rb/R)**3./R
+            return 4.*self._af*smooth*numpy.cos(2.*(phi-self._omegab*t-
+                                                    self._barphi))\
+                                                 *(self._rb/r)**3.*R**2./r**2.
 
+    def _Rphideriv(self,R,z,phi=0.,t=0.):
+        #Calculate relevant time
+        if t < self._tform:
+            smooth= 0.
+        elif t < self._tsteady:
+            deltat= t-self._tform
+            xi= 2.*deltat/(self._tsteady-self._tform)-1.
+            smooth= (3./16.*xi**5.-5./8*xi**3.+15./16.*xi+.5)
+        else: #bar is fully on
+            smooth= 1.
+        r= numpy.sqrt(R**2.+z**2.)
+        if r <= self._rb:
+            return -2.*self._af*smooth*numpy.sin(2.*(phi-self._omegab*t
+                                                  -self._barphi))\
+                    *((r/self._rb)**3.*R*(3.*R**2.+2.*z**2.)-4.*R*z**2.)/r**4.
+        else:
+            return -2.*self._af*smooth*numpy.sin(2.*(phi-self._omegab*t-
+                                                  self._barphi))\
+                    *(self._rb/r)**3.*R/r**4.*(3.*R**2.-2.*z**2.)
+
+    def _z2deriv(self,R,z,phi=0.,t=0.):
+        #Calculate relevant time
+        if t < self._tform:
+            smooth= 0.
+        elif t < self._tsteady:
+            deltat= t-self._tform
+            xi= 2.*deltat/(self._tsteady-self._tform)-1.
+            smooth= (3./16.*xi**5.-5./8*xi**3.+15./16.*xi+.5)
+        else: #bar is fully on
+            smooth= 1.
+        r= numpy.sqrt(R**2.+z**2.)
+        if r <= self._rb:
+            return self._af*smooth*numpy.cos(2.*(phi-self._omegab*t
+                                                  -self._barphi))\
+                   *R**2./r**6.*((r/self._rb)**3.*(r**2.-z**2.)
+                                 +4.*(r**2.-4.*z**2.))
+        else:
+            return 5.*self._af*smooth*numpy.cos(2.*(phi-self._omegab*t-
+                                                  self._barphi))\
+                    *(self._rb/r)**3.*R**2./r**6.*(r**2.-7.*z**2.)
+        
+    def _Rzderiv(self,R,z,phi=0.,t=0.):
+        #Calculate relevant time
+        if t < self._tform:
+            smooth= 0.
+        elif t < self._tsteady:
+            deltat= t-self._tform
+            xi= 2.*deltat/(self._tsteady-self._tform)-1.
+            smooth= (3./16.*xi**5.-5./8*xi**3.+15./16.*xi+.5)
+        else: #bar is fully on
+            smooth= 1.
+        r= numpy.sqrt(R**2.+z**2.)
+        if r <= self._rb:
+            return self._af*smooth*numpy.cos(2.*(phi-self._omegab*t
+                                                  -self._barphi))\
+                   *R*z/r**6.*((r/self._rb)**3.*(2.*r**2.-R**2.)
+                                 +8.*(r**2.-2.*R**2.))
+        else:
+            return 5.*self._af*smooth*numpy.cos(2.*(phi-self._omegab*t-
+                                                  self._barphi))\
+                    *(self._rb/r)**3.*R*z/r**6.*(2.*r**2.-7.*R**2.)
+        
     def tform(self): #pragma: no cover
         """
         NAME:
