@@ -22,6 +22,7 @@ import math
 import numpy as nu
 from scipy import optimize, integrate
 import galpy.util.bovy_plot as plot
+from galpy.util import bovy_coords
 from galpy.util import config
 from galpy.util.bovy_conversion import velocity_in_kpcGyr, \
     physical_conversion, potential_physical_input, freq_in_Gyr
@@ -488,9 +489,10 @@ class Potential(object):
                                            *self.dens(x,y,use_physical=False),
                                        0.,R,lambda x: 0., lambda x: z)[0]
 
-    @physical_conversion('mass',pop=True)
+    @physical_conversion('mass',pop=False)
     def mvir(self,H=70.,Om=0.3,overdens=200.,wrtcrit=False,
-             forceint=False):
+             forceint=False,ro=None,vo=None,
+             use_physical=False): # use_physical necessary bc of pop=False, does nothing inside
         """
         NAME:
 
@@ -527,13 +529,15 @@ class Potential(object):
            2014-09-12 - Written - Bovy (IAS)
 
         """
+        if ro is None: ro= self._ro
+        if vo is None: vo= self._vo
         #Evaluate the virial radius
         try:
             rvir= self.rvir(H=H,Om=Om,overdens=overdens,wrtcrit=wrtcrit,
-                            use_physical=False)
+                            use_physical=False,ro=ro,vo=vo)
         except AttributeError:
             raise AttributeError("This potential does not have a '_scale' defined to base the concentration on or does not support calculating the virial radius")
-        return self.mass(rvir,forceint=forceint,use_physical=False)
+        return self.mass(rvir,forceint=forceint,use_physical=False,ro=ro,vo=vo)
 
     @potential_physical_input
     @physical_conversion('forcederivative',pop=True)
@@ -843,9 +847,9 @@ class Potential(object):
         return RZToverticalPotential(self,R)
 
     def plot(self,t=0.,rmin=0.,rmax=1.5,nrs=21,zmin=-0.5,zmax=0.5,nzs=21,
-             effective=False,Lz=None,phi=None,
+             effective=False,Lz=None,phi=None,xy=False,
              xrange=None,yrange=None,
-             justcontours=False,
+             justcontours=False,levels=None,cntrcolors=None,
              ncontours=21,savefilename=None):
         """
         NAME:
@@ -858,34 +862,40 @@ class Potential(object):
 
         INPUT:
 
-           t= time tp plot potential at
+           t= time to plot potential at
 
-           rmin= minimum R at which to calculate (can be Quantity)
+           rmin= minimum R (can be Quantity) [xmin if xy]
 
-           rmax= maximum R (can be Quantity)
+           rmax= maximum R (can be Quantity) [ymax if xy]
 
            nrs= grid in R
 
-           zmin= minimum z (can be Quantity)
+           zmin= minimum z (can be Quantity) [ymin if xy]
 
-           zmax= maximum z (can be Quantity)
+           zmax= maximum z (can be Quantity) [ymax if xy]
 
            nzs= grid in z
 
            phi= (None) azimuth to use for non-axisymmetric potentials
 
+           xy= (False) if True, plot the potential in X-Y
+
            effective= (False) if True, plot the effective potential Phi + Lz^2/2/R^2
 
            Lz= (None) angular momentum to use for the effective potential when effective=True
 
-           ncontours - number of contours
-
            justcontours= (False) if True, just plot contours
 
            savefilename - save to or restore from this savefile (pickle)
 
            xrange, yrange= can be specified independently from rmin,zmin, etc.
 
+           levels= (None) contours to plot
+
+           ncontours - number of contours when levels is None
+
+           cntrcolors= (None) colors of the contours (single color or array with length ncontours)
+
         OUTPUT:
 
            plot to output device
@@ -923,8 +933,12 @@ class Potential(object):
             potRz= nu.zeros((nrs,nzs))
             for ii in range(nrs):
                 for jj in range(nzs):
+                    if xy:
+                        R,phi,z= bovy_coords.rect_to_cyl(Rs[ii],zs[jj],0.)
+                    else:
+                        R,z= Rs[ii], zs[jj]
                     potRz[ii,jj]= evaluatePotentials(self,
-                                                     Rs[ii],zs[jj],t=t,phi=phi,
+                                                     R,z,t=t,phi=phi,
                                                      use_physical=False)
                 if effective:
                     potRz[ii,:]+= 0.5*Lz**2/Rs[ii]**2.
@@ -940,18 +954,28 @@ class Potential(object):
                 pickle.dump(Rs,savefile)
                 pickle.dump(zs,savefile)
                 savefile.close()
+        if xy:
+            xlabel= r'$x/R_0$'
+            ylabel= r'$y/R_0$'
+        else:
+            xlabel=r"$R/R_0$"
+            ylabel=r"$z/R_0$"
+        if levels is None:
+            levels= nu.linspace(nu.nanmin(potRz),nu.nanmax(potRz),ncontours)
+        if cntrcolors is None:
+            cntrcolors= 'k'
         return plot.bovy_dens2d(potRz.T,origin='lower',cmap='gist_gray',contours=True,
-                                xlabel=r"$R/R_0$",ylabel=r"$z/R_0$",
+                                xlabel=xlabel,ylabel=ylabel,
                                 xrange=xrange,
                                 yrange=yrange,
                                 aspect=.75*(rmax-rmin)/(zmax-zmin),
                                 cntrls='-',
                                 justcontours=justcontours,
-                                levels=nu.linspace(nu.nanmin(potRz),nu.nanmax(potRz),
-                                                   ncontours))
+                                levels=levels,cntrcolors=cntrcolors)
         
-    def plotDensity(self,rmin=0.,rmax=1.5,nrs=21,zmin=-0.5,zmax=0.5,nzs=21,
-                    phi=None,
+    def plotDensity(self,t=0.,
+                    rmin=0.,rmax=1.5,nrs=21,zmin=-0.5,zmax=0.5,nzs=21,
+                    phi=None,xy=False,
                     ncontours=21,savefilename=None,aspect=None,log=False,
                     justcontours=False):
         """
@@ -965,20 +989,24 @@ class Potential(object):
 
         INPUT:
 
-           rmin= minimum R (can be Quantity)
+           t= time to plot potential at
 
-           rmax= maximum R (can be Quantity)
+           rmin= minimum R (can be Quantity) [xmin if xy]
+
+           rmax= maximum R (can be Quantity) [ymax if xy]
 
            nrs= grid in R
 
-           zmin= minimum z (can be Quantity)
+           zmin= minimum z (can be Quantity) [ymin if xy]
 
-           zmax= maximum z (can be Quantity)
+           zmax= maximum z (can be Quantity) [ymax if xy]
 
            nzs= grid in z
 
            phi= (None) azimuth to use for non-axisymmetric potentials
 
+           xy= (False) if True, plot the density in X-Y
+
            ncontours= number of contours
 
            justcontours= (False) if True, just plot contours
@@ -997,7 +1025,7 @@ class Potential(object):
 
         """
         return plotDensities(self,rmin=rmin,rmax=rmax,nrs=nrs,
-                             zmin=zmin,zmax=zmax,nzs=nzs,phi=phi,
+                             zmin=zmin,zmax=zmax,nzs=nzs,phi=phi,xy=xy,t=t,
                              ncontours=ncontours,savefilename=savefilename,
                              justcontours=justcontours,
                              aspect=aspect,log=log)
@@ -1315,7 +1343,7 @@ class Potential(object):
             sinl= nu.sin(l/180.*nu.pi)
         else:
             sinl= nu.sin(l)
-        return sinl*(self.omegac(sinl,use_physical=False)\
+        return sinl*(self.omegac(nu.fabs(sinl),use_physical=False)\
                          -self.omegac(1.,use_physical=False))
 
     def plotRotcurve(self,*args,**kwargs):
@@ -1382,7 +1410,8 @@ class Potential(object):
         """
         return plotEscapecurve(self.toPlanar(),*args,**kwargs)
 
-    def conc(self,H=70.,Om=0.3,overdens=200.,wrtcrit=False):
+    def conc(self,H=70.,Om=0.3,overdens=200.,wrtcrit=False,
+             ro=None,vo=None):
         """
         NAME:
 
@@ -1415,9 +1444,11 @@ class Potential(object):
            2014-04-03 - Written - Bovy (IAS)
 
         """
+        if ro is None: ro= self._ro
+        if vo is None: vo= self._vo
         try:
             return self.rvir(H=H,Om=Om,overdens=overdens,wrtcrit=wrtcrit,
-                             use_physical=False)/self._scale
+                             ro=ro,vo=vo,use_physical=False)/self._scale
         except AttributeError:
             raise AttributeError("This potential does not have a '_scale' defined to base the concentration on or does not support calculating the virial radius")
 
@@ -1932,8 +1963,9 @@ def evaluateRzderivs(Pot,R,z,phi=None,t=0.):
         raise PotentialError("Input to 'evaluateRzderivs' is neither a Potential-instance or a list of such instances")
 
 def plotPotentials(Pot,rmin=0.,rmax=1.5,nrs=21,zmin=-0.5,zmax=0.5,nzs=21,
-                   phi=None,ncontours=21,savefilename=None,aspect=None,
-                   justcontours=False):
+                   phi=None,xy=False,t=0.,effective=False,Lz=None,
+                   ncontours=21,savefilename=None,aspect=None,
+                   justcontours=False,levels=None,cntrcolors=None):
         """
         NAME:
 
@@ -1947,24 +1979,36 @@ def plotPotentials(Pot,rmin=0.,rmax=1.5,nrs=21,zmin=-0.5,zmax=0.5,nzs=21,
 
            Pot - Potential or list of Potential instances
 
-           rmin= minimum R (can be Quantity)
+           rmin= minimum R (can be Quantity) [xmin if xy]
 
-           rmax= maximum R (can be Quantity)
+           rmax= maximum R (can be Quantity) [ymax if xy]
 
            nrs= grid in R
 
-           zmin= minimum z (can be Quantity)
+           zmin= minimum z (can be Quantity) [ymin if xy]
 
-           zmax= maximum z (can be Quantity)
+           zmax= maximum z (can be Quantity) [ymax if xy]
 
            nzs= grid in z
 
            phi= (None) azimuth to use for non-axisymmetric potentials
 
-           ncontours= number of contours
+           t= (0.) time to use to evaluate potential
+
+           xy= (False) if True, plot the potential in X-Y
+
+           effective= (False) if True, plot the effective potential Phi + Lz^2/2/R^2
+
+           Lz= (None) angular momentum to use for the effective potential when effective=True
 
            justcontours= (False) if True, just plot contours
 
+           levels= (None) contours to plot
+
+           ncontours - number of contours when levels is None
+
+           cntrcolors= (None) colors of the contours (single color or array with length ncontours)
+
            savefilename= save to or restore from this savefile (pickle)
 
         OUTPUT:
@@ -1997,14 +2041,22 @@ def plotPotentials(Pot,rmin=0.,rmax=1.5,nrs=21,zmin=-0.5,zmax=0.5,nzs=21,
             zs= pickle.load(savefile)
             savefile.close()
         else:
+            if effective and Lz is None:
+                raise RuntimeError("When effective=True, you need to specify Lz=")
             Rs= nu.linspace(rmin,rmax,nrs)
             zs= nu.linspace(zmin,zmax,nzs)
             potRz= nu.zeros((nrs,nzs))
             for ii in range(nrs):
                 for jj in range(nzs):
-                    potRz[ii,jj]= evaluatePotentials(Pot,nu.fabs(Rs[ii]),
-                                                     zs[jj],phi=phi,
+                    if xy:
+                        R,phi,z= bovy_coords.rect_to_cyl(Rs[ii],zs[jj],0.)
+                    else:
+                        R,z= Rs[ii], zs[jj]
+                    potRz[ii,jj]= evaluatePotentials(Pot,nu.fabs(R),
+                                                     z,phi=phi,t=t,
                                                      use_physical=False)
+                if effective:
+                    potRz[ii,:]+= 0.5*Lz**2/Rs[ii]**2.
             if not savefilename == None:
                 print("Writing savefile "+savefilename+" ...")
                 savefile= open(savefilename,'wb')
@@ -2014,18 +2066,27 @@ def plotPotentials(Pot,rmin=0.,rmax=1.5,nrs=21,zmin=-0.5,zmax=0.5,nzs=21,
                 savefile.close()
         if aspect is None:
             aspect=.75*(rmax-rmin)/(zmax-zmin)
+        if xy:
+            xlabel= r'$x/R_0$'
+            ylabel= r'$y/R_0$'
+        else:
+            xlabel=r"$R/R_0$"
+            ylabel=r"$z/R_0$"
+        if levels is None:
+            levels= nu.linspace(nu.nanmin(potRz),nu.nanmax(potRz),ncontours)
+        if cntrcolors is None:
+            cntrcolors= 'k'
         return plot.bovy_dens2d(potRz.T,origin='lower',cmap='gist_gray',contours=True,
-                                xlabel=r"$R/R_0$",ylabel=r"$z/R_0$",
+                                xlabel=xlabel,ylabel=ylabel,
                                 aspect=aspect,
                                 xrange=[rmin,rmax],
                                 yrange=[zmin,zmax],
                                 cntrls='-',
                                 justcontours=justcontours,
-                                levels=nu.linspace(nu.nanmin(potRz),nu.nanmax(potRz),
-                                                   ncontours))
+                                levels=levels,cntrcolors=cntrcolors)
 
 def plotDensities(Pot,rmin=0.,rmax=1.5,nrs=21,zmin=-0.5,zmax=0.5,nzs=21,
-                  phi=None,
+                  phi=None,xy=False,t=0.,
                   ncontours=21,savefilename=None,aspect=None,log=False,
                   justcontours=False):
         """
@@ -2041,20 +2102,24 @@ def plotDensities(Pot,rmin=0.,rmax=1.5,nrs=21,zmin=-0.5,zmax=0.5,nzs=21,
 
            Pot - Potential or list of Potential instances
 
-           rmin= minimum R (can be Quantity)
+           rmin= minimum R (can be Quantity) [xmin if xy]
 
-           rmax= maximum R (can be Quantity)
+           rmax= maximum R (can be Quantity) [ymax if xy]
 
            nrs= grid in R
 
-           zmin= minimum z (can be Quantity)
+           zmin= minimum z (can be Quantity) [ymin if xy]
 
-           zmax= maximum z (can be Quantity)
+           zmax= maximum z (can be Quantity) [ymax if xy]
 
            nzs= grid in z
 
            phi= (None) azimuth to use for non-axisymmetric potentials
 
+           t= (0.) time to use to evaluate potential
+
+           xy= (False) if True, plot the density in X-Y
+
            ncontours= number of contours
 
            justcontours= (False) if True, just plot contours
@@ -2098,8 +2163,12 @@ def plotDensities(Pot,rmin=0.,rmax=1.5,nrs=21,zmin=-0.5,zmax=0.5,nzs=21,
             potRz= nu.zeros((nrs,nzs))
             for ii in range(nrs):
                 for jj in range(nzs):
-                    potRz[ii,jj]= evaluateDensities(Pot,nu.fabs(Rs[ii]),
-                                                    zs[jj],phi=phi,
+                    if xy:
+                        R,phi,z= bovy_coords.rect_to_cyl(Rs[ii],zs[jj],0.)
+                    else:
+                        R,z= Rs[ii], zs[jj]
+                    potRz[ii,jj]= evaluateDensities(Pot,nu.fabs(R),z,phi=phi,
+                                                    t=t,
                                                     use_physical=False)
             if not savefilename == None:
                 print("Writing savefile "+savefilename+" ...")
@@ -2112,8 +2181,15 @@ def plotDensities(Pot,rmin=0.,rmax=1.5,nrs=21,zmin=-0.5,zmax=0.5,nzs=21,
             aspect=.75*(rmax-rmin)/(zmax-zmin)
         if log:
             potRz= nu.log(potRz)
-        return plot.bovy_dens2d(potRz.T,origin='lower',cmap='gist_yarg',contours=True,
-                                xlabel=r"$R/R_0$",ylabel=r"$z/R_0$",
+        if xy:
+            xlabel= r'$x/R_0$'
+            ylabel= r'$y/R_0$'
+        else:
+            xlabel=r"$R/R_0$"
+            ylabel=r"$z/R_0$"
+        return plot.bovy_dens2d(potRz.T,origin='lower',
+                                cmap='gist_yarg',contours=True,
+                                xlabel=xlabel,ylabel=ylabel,
                                 aspect=aspect,
                                 xrange=[rmin,rmax],
                                 yrange=[zmin,zmax],
@@ -2576,7 +2652,7 @@ def turn_physical_on(Pot,ro=None,vo=None):
         Pot.turn_physical_on(ro=ro,vo=vo)
     return None
 
-def _check_c(Pot):
+def _check_c(Pot,dxdv=False):
     """
 
     NAME:
@@ -2591,6 +2667,8 @@ def _check_c(Pot):
 
        Pot - Potential instance or list of such instances
 
+       dxdv= (False) check whether the potential has dxdv implementation
+
     OUTPUT:
 
        True if a C implementation exists, False otherwise
@@ -2599,11 +2677,20 @@ def _check_c(Pot):
 
        2014-02-17 - Written - Bovy (IAS)
 
+       2017-07-01 - Generalized to dxdv, added general support for WrapperPotentials, and added support for planarPotentials
+
     """
+    from galpy.potential import planarPotential
+    if dxdv: hasC_attr= 'hasC_dxdv'
+    else: hasC_attr= 'hasC'
+    from galpy.potential_src.WrapperPotential import parentWrapperPotential
     if isinstance(Pot,list):
-        return nu.all(nu.array([p.hasC for p in Pot],dtype='bool'))
-    elif isinstance(Pot,Potential):
-        return Pot.hasC
+        return nu.all(nu.array([_check_c(p,dxdv=dxdv) for p in Pot],
+                               dtype='bool'))
+    elif isinstance(Pot,parentWrapperPotential):
+        return bool(Pot.__dict__[hasC_attr]*_check_c(Pot._pot))
+    elif isinstance(Pot,Potential) or isinstance(Pot,planarPotential):
+        return Pot.__dict__[hasC_attr]
 
 def _dim(Pot):
     """
