@@ -4,6 +4,9 @@
 #include <gsl/gsl_sf_gegenbauer.h>
 #include <gsl/gsl_sf_legendre.h>
 
+#ifndef GSL_MAJOR_VERSION
+#define GSL_MAJOR_VERSION 1
+#endif
 //SCF Disk potential
 //4 arguments: amp, Acos, Asin, a
 
@@ -13,7 +16,7 @@ const int DERIV =2;
 //Useful Functions
 
 //Converts from cylindrical coordinates to spherical
-inline void cyl_to_spher(double R, double Z,double *r, double *theta)
+static inline void cyl_to_spher(double R, double Z,double *r, double *theta)
 {
     *r = sqrt(R*R + Z*Z);
     *theta = atan2(R, Z);
@@ -28,7 +31,7 @@ double power(double x, int i)
 }
 
 //Calculates xi
-inline void calculateXi(double r, double a, double *xi)
+static inline void calculateXi(double r, double a, double *xi)
 {
     *xi = (r - a)/(r + a);
 }
@@ -101,7 +104,7 @@ double computeAxiF_phiphi(double Acos_val, double P, double d2phiTilde)
 }
 
 //Calculates the Gegenbauer polynomials
-inline void compute_C(double xi, int N, int L, double * C_array)
+void compute_C(double xi, int N, int L, double * C_array)
 {
     int l;
     for (l = 0; l < L; l++)
@@ -111,7 +114,7 @@ inline void compute_C(double xi, int N, int L, double * C_array)
 }
 
 //Calculates the derivative of the Gegenbauer polynomials
-inline void compute_dC(double xi, int N, int L, double * dC_array)
+void compute_dC(double xi, int N, int L, double * dC_array)
 {
     int n,l;
     for (l = 0; l < L; l++)
@@ -128,7 +131,7 @@ inline void compute_dC(double xi, int N, int L, double * dC_array)
 }
 
 //Calculates the second derivative of the Gegenbauer polynomials
-inline void compute_d2C(double xi, int N, int L, double * d2C_array)
+void compute_d2C(double xi, int N, int L, double * d2C_array)
 {
     int n,l;
     for (l = 0; l < L; l++)
@@ -147,7 +150,7 @@ inline void compute_d2C(double xi, int N, int L, double * d2C_array)
 }
 
 //Compute phi_Tilde
-inline void compute_phiTilde(double r, double a, int N, int L, double* C, double * phiTilde)
+void compute_phiTilde(double r, double a, int N, int L, double* C, double * phiTilde)
 {
     double xi;
     calculateXi(r, a, &xi);
@@ -168,7 +171,7 @@ inline void compute_phiTilde(double r, double a, int N, int L, double* C, double
 }
 
 //Computes the derivative of phiTilde with respect to r
-inline void compute_dphiTilde(double r, double a, int N, int L, double * C, double * dC, double * dphiTilde)
+void compute_dphiTilde(double r, double a, int N, int L, double * C, double * dC, double * dphiTilde)
 {
     double xi;
     calculateXi(r, a, &xi);
@@ -190,7 +193,7 @@ inline void compute_dphiTilde(double r, double a, int N, int L, double * C, doub
 }
 
 //Computes the second derivative of phiTilde with respect to r
-inline void compute_d2phiTilde(double r, double a, int N, int L, double * C, double * dC,double * d2C, double * d2phiTilde)
+void compute_d2phiTilde(double r, double a, int N, int L, double * C, double * dC,double * d2C, double * d2phiTilde)
 {
     double xi;
     calculateXi(r, a, &xi);
@@ -219,31 +222,54 @@ inline void compute_d2phiTilde(double r, double a, int N, int L, double * C, dou
     }
 }
 
+
 //Computes the associated Legendre polynomials
-inline void compute_P(double x, int L, int M, double * P_array)
+void compute_P(double x, int L, int M, double * P_array)
 {
-    int m;
-    for (m = 0; m < M; m++)
-    {
-        int shift = m*L + m;
-        gsl_sf_legendre_Plm_array(L - 1, m, x, P_array + shift);
+    if (M == 1){
+        gsl_sf_legendre_Pl_array (L - 1, x, P_array);
+    } else {
+        #if GSL_MAJOR_VERSION == 2
+            gsl_sf_legendre_array_e(GSL_SF_LEGENDRE_NONE,L - 1, x, -1, P_array);
+        #else
+            int m;
+            for (m = 0; m < M; m++)
+            {
+                gsl_sf_legendre_Plm_array(L - 1, m, x, P_array);
+                P_array += L - m;
+            }
+        #endif
     }
+    
+      
+    
 }
 
 //Computes the associated Legendre polynomials and its derivative
-inline void compute_P_dP(double x, int L, int M, double * P_array, double *dP_array)
+void compute_P_dP(double x, int L, int M, double * P_array, double *dP_array)
 {
-    int m;
-    for (m = 0; m < M; m++)
-    {
-        int shift = m*L + m;
-        gsl_sf_legendre_Plm_deriv_array(L - 1, m, x, P_array + shift, dP_array + shift);
+    if (M == 1){
+        gsl_sf_legendre_Pl_deriv_array (L - 1, x, P_array, dP_array);
+    } else {
+        #if GSL_MAJOR_VERSION == 2
+            gsl_sf_legendre_deriv_array_e(GSL_SF_LEGENDRE_NONE, L - 1, x, -1,P_array, dP_array);
+        
+        #else
+            int m;
+            for (m = 0; m < M; m++)
+            {
+                gsl_sf_legendre_Plm_deriv_array(L - 1, m, x, P_array, dP_array);
+                P_array += L - m;
+                dP_array += L - m;
+            }
+        #endif
     }
 }
 
 
 
 
+
 typedef struct equations equations;
 struct equations
 {
@@ -263,11 +289,11 @@ struct axi_equations
 };
 
 //Compute axi symmetric potentials.
-inline void compute(double a, int N, int L, int M,
-                    double r, double theta, double phi,
-                    double *Acos, int eq_size,
-                    axi_equations e,
-                    double *F)
+void compute(double a, int N, int L, int M,
+	     double r, double theta, double phi,
+	     double *Acos, int eq_size,
+	     axi_equations e,
+	     double *F)
 {
     int i,n,l;
     for (i = 0; i < eq_size; i++)
@@ -306,11 +332,11 @@ inline void compute(double a, int N, int L, int M,
 }
 
 //Compute Non Axi symmetric Potentials
-inline void computeNonAxi(double a, int N, int L, int M,
-                          double r, double theta, double phi,
-                          double *Acos, double *Asin, int eq_size,
-                          equations e,
-                          double *F)
+void computeNonAxi(double a, int N, int L, int M,
+		   double r, double theta, double phi,
+		   double *Acos, double *Asin, int eq_size,
+		   equations e,
+		   double *F)
 {
     int i,n,l,m;
     for (i = 0; i < eq_size; i++)
@@ -318,11 +344,20 @@ inline void computeNonAxi(double a, int N, int L, int M,
         *(F + i) =0; //Initialize each F
     }
 
-
+    int v1counter=0;
+    int v2counter = 0;
+    int Pindex = 0;
     for (l = 0; l < L; l++)
     {
+        v1counter = 0;
         for (m = 0; m<=l; m++)
         {
+            #if GSL_MAJOR_VERSION == 2
+                Pindex = v2counter;
+            #else
+                Pindex = v1counter + l;
+            #endif
+            
             double mCos = cos(m*phi);
             double mSin = sin(m*phi);
             for (n = 0; n < N; n++)
@@ -334,12 +369,22 @@ inline void computeNonAxi(double a, int N, int L, int M,
                     double (*Eq)(double, double, double, double, double, double, int) = *(e.Eq + i);
                     double *P = *(e.P + i);
                     double *phiTilde = *(e.phiTilde + i);
-                    *(F + i) += (*Eq)(Acos_val, Asin_val, mCos, mSin, P[m*L + l], phiTilde[l*N + n], m);
+                    *(F + i) += (*Eq)(Acos_val, Asin_val, mCos, mSin, P[Pindex], phiTilde[l*N + n], m);
                 }
 
 
             }
+            
+           
+            
+                v2counter++; 
+                v1counter+=M-m - 1;  
+                
         }
+        
+        
+            
+        
     }
     //Multiply F by constants
     for (i = 0; i < eq_size; i++)
@@ -397,41 +442,48 @@ void computeForce(double R,double Z, double phi,
     double C[N*L];
     double dC[N*L];
 
-    compute_C(xi, N, L, &C);
-    compute_dC(xi, N, L, &dC);
+    compute_C(xi, N, L, &C[0]);
+    compute_dC(xi, N, L, &dC[0]);
 
 //Compute phiTilde and its derivative
     double phiTilde[L*N];
-    compute_phiTilde(r, a, N, L, &C, &phiTilde);
+    compute_phiTilde(r, a, N, L, &C[0], &phiTilde[0]);
 
     double dphiTilde[L*N];
-    compute_dphiTilde(r, a, N, L, &C, &dC, &dphiTilde);
+    compute_dphiTilde(r, a, N, L, &C[0], &dC[0], &dphiTilde[0]);
 
 //Compute Associated Legendre Polynomials
     int M_eff = M;
+    int size = 0;
+    
     if (isNonAxi==0)
     {
-        M_eff = 1;
+    M_eff = 1;
+    size = L;    
+    } else{
+    size = L*L - L*(L-1)/2;
     }
-    double P[M_eff*L];
-    double dP[M_eff*L];
-    compute_P_dP(cos(theta), L, M_eff, &P, &dP);
+    
+    
+    double P[size];
+    double dP[size];
+    compute_P_dP(cos(theta), L, M_eff, &P[0], &dP[0]);
 
-    double (*PhiTilde_Pointer[3]) = {&dphiTilde, &phiTilde, &phiTilde};
-    double (*P_Pointer[3]) = {&P, &dP, &P};
+    double (*PhiTilde_Pointer[3]) = {&dphiTilde[0],&phiTilde[0],&phiTilde[0]};
+    double (*P_Pointer[3]) = {&P[0], &dP[0], &P[0]};
 
     double Constant[3] = {1., -sin(theta), 1.};
 
     if (isNonAxi == 1)
     {
         double (*Eq[3])(double, double, double, double, double, double, int) = {&computeF_r, &computeF_theta, &computeF_phi};
-        equations e = {Eq,&PhiTilde_Pointer, &P_Pointer, &Constant};
+        equations e = {Eq,&PhiTilde_Pointer[0], &P_Pointer[0], &Constant[0]};
         computeNonAxi(a, N, L, M,r, theta, phi, Acos, Asin, 3, e, F);
     }
     else
     {
         double (*Eq[3])(double, double, double) = {&computeAxiF_r, &computeAxiF_theta, &computeAxiF_phi};
-        axi_equations e = {Eq,&PhiTilde_Pointer, &P_Pointer, &Constant};
+        axi_equations e = {Eq,&PhiTilde_Pointer[0], &P_Pointer[0], &Constant[0]};
         compute(a, N, L, M,r, theta, phi, Acos, 3, e, F);
     }
 
@@ -499,52 +551,51 @@ void computeDeriv(double R,double Z, double phi,
     double dC[N*L];
     double d2C[N*L];
 
-    compute_C(xi, N, L, &C);
-    compute_dC(xi, N, L, &dC);
-    compute_d2C(xi, N, L, &d2C);
+    compute_C(xi, N, L, &C[0]);
+    compute_dC(xi, N, L, &dC[0]);
+    compute_d2C(xi, N, L, &d2C[0]);
 
 //Compute phiTilde and its derivative
     double phiTilde[L*N];
-    compute_phiTilde(r, a, N, L, &C, &phiTilde);
+    compute_phiTilde(r, a, N, L, &C[0], &phiTilde[0]);
 
     double dphiTilde[L*N];
-    compute_dphiTilde(r, a, N, L, &C, &dC, &dphiTilde);
+    compute_dphiTilde(r, a, N, L, &C[0], &dC[0], &dphiTilde[0]);
 
     double d2phiTilde[L*N];
-    compute_d2phiTilde(r, a, N, L, &C, &dC, &d2C, &d2phiTilde);
+    compute_d2phiTilde(r, a, N, L, &C[0], &dC[0], &d2C[0], &d2phiTilde[0]);
 
 
 //Compute Associated Legendre Polynomials
     int M_eff = M;
+    int size = 0;
+    
     if (isNonAxi==0)
     {
-        M_eff = 1;
+    M_eff = 1;
+    size = L;    
+    } else{
+    size = L*L - L*(L-1)/2;
     }
-    double P[M_eff*L];
+    double P[size];
 
+    compute_P(cos(theta), L,M_eff, &P[0]);
 
-    compute_P(cos(theta), L,M_eff, &P);
-
-
-
-    int num_eq = 3;
-
-
-    double (*PhiTilde_Pointer[3]) = {&d2phiTilde, &phiTilde, &dphiTilde};
-    double (*P_Pointer[3]) = {&P, &P, &P};
+    double (*PhiTilde_Pointer[3])= {&d2phiTilde[0],&phiTilde[0],&dphiTilde[0]};
+    double (*P_Pointer[3]) = {&P[0], &P[0], &P[0]};
 
     double Constant[3] = {1., 1., 1.};
 
     if (isNonAxi==1)
     {
         double (*Eq[3])(double, double, double, double, double, double, int) = {&computeF_rr, &computeF_phiphi, &computeF_rphi};
-        equations e = {Eq,&PhiTilde_Pointer, &P_Pointer, &Constant};
+        equations e = {Eq,&PhiTilde_Pointer[0],&P_Pointer[0],&Constant[0]};
         computeNonAxi(a, N, L, M,r, theta, phi, Acos, Asin, 3, e, F);
     }
     else
     {
         double (*Eq[3])(double, double, double) = {&computeAxiF_rr, &computeAxiF_phiphi, &computeAxiF_rphi};
-        axi_equations e = {Eq,&PhiTilde_Pointer, &P_Pointer, &Constant};
+        axi_equations e = {Eq,&PhiTilde_Pointer[0],&P_Pointer[0],&Constant[0]};
         compute(a, N, L, M,r, theta, phi, Acos, 3, e, F);
     }
 
@@ -590,43 +641,46 @@ double SCFPotentialEval(double R,double Z, double phi,
     //Compute the gegenbauer polynomials and its derivative.
     double C[N*L];
 
-    compute_C(xi, N, L, &C);
+    compute_C(xi, N, L, &C[0]);
 
     //Compute phiTilde and its derivative
     double phiTilde[L*N];
-    compute_phiTilde(r, a, N, L, &C, &phiTilde);
+    compute_phiTilde(r, a, N, L, &C[0], &phiTilde[0]);
     //Compute Associated Legendre Polynomials
 
     int M_eff = M;
+    int size = 0;
+    
     if (isNonAxi==0)
     {
-        M_eff = 1;
-    }
-    double P[M_eff*L];
+    M_eff = 1;
+    size = L;    
+    } else{ //LCOV_EXCL_START
+    size = L*L - L*(L-1)/2;
+    } //LCOV_EXCL_STOP
+    
+    double P[size];
 
 
-    compute_P(cos(theta), L,M_eff, &P);
+    compute_P(cos(theta), L,M_eff, &P[0]);
 
     double potential;
 
-    int num_eq = 1;
-
-
-    double (*PhiTilde_Pointer[1]) = {&phiTilde};
-    double (*P_Pointer[1]) = {&P};
+    double (*PhiTilde_Pointer[1]) = {&phiTilde[0]};
+    double (*P_Pointer[1]) = {&P[0]};
 
     double Constant[1] = {1.};
 
     if (isNonAxi==1) //LCOV_EXCL_START
     {
         double (*Eq[1])(double, double, double, double, double, double, int) = {&computePhi};
-        equations e = {Eq,&PhiTilde_Pointer, &P_Pointer, &Constant};
+        equations e = {Eq,&PhiTilde_Pointer[0],&P_Pointer[0],&Constant[0]};
         computeNonAxi(a, N, L, M,r, theta, phi, Acos, Asin, 1, e, &potential);
     } //LCOV_EXCL_STOP
     else
     {
         double (*Eq[1])(double, double, double) = {&computeAxiPhi};
-        axi_equations e = {Eq,&PhiTilde_Pointer, &P_Pointer, &Constant};
+        axi_equations e = {Eq,&PhiTilde_Pointer[0],&P_Pointer[0],&Constant[0]};
         compute(a, N, L, M,r, theta, phi, Acos, 1, e, &potential);
     }
 
@@ -648,7 +702,7 @@ double SCFPotentialRforce(double R,double Z, double phi,
 
 
     double F[3];
-    computeForce(R, Z, phi, t,potentialArgs, &F) ;
+    computeForce(R, Z, phi, t,potentialArgs, &F[0]) ;
 
     return *(F + 0)*dr_dR + *(F + 1)*dtheta_dR + *(F + 2)*dphi_dR;
 
@@ -669,7 +723,7 @@ double SCFPotentialzforce(double R,double Z, double phi,
     double dphi_dz = 0;
 
     double F[3];
-    computeForce(R, Z, phi, t,potentialArgs, &F) ;
+    computeForce(R, Z, phi, t,potentialArgs, &F[0]) ;
     return *(F + 0)*dr_dz + *(F + 1)*dtheta_dz + *(F + 2)*dphi_dz;
 }
 
@@ -688,7 +742,7 @@ double SCFPotentialphiforce(double R,double Z, double phi,
     double dphi_dphi = 1;
 
     double F[3];
-    computeForce(R, Z, phi, t,potentialArgs, &F) ;
+    computeForce(R, Z, phi, t,potentialArgs, &F[0]) ;
 
     return *(F + 0)*dr_dphi + *(F + 1)*dtheta_dphi + *(F + 2)*dphi_dphi;
 }
@@ -698,7 +752,6 @@ double SCFPotentialPlanarRforce(double R,double phi,
                                 double t,
                                 struct potentialArg * potentialArgs)
 {
-    double * args= potentialArgs->args;
     return SCFPotentialRforce(R,0., phi,t,potentialArgs);
 
 }
@@ -717,14 +770,9 @@ double SCFPotentialPlanarR2deriv(double R, double phi,
                                  double t,
                                  struct potentialArg * potentialArgs)
 {
-
     double Farray[3];
-
-    computeDeriv(R, 0, phi, t,potentialArgs, &Farray) ;
-
-    return *(Farray + 0);
-
-
+    computeDeriv(R, 0, phi, t,potentialArgs, &Farray[0]) ;
+    return *Farray;
 }
 
 //Compute the planar double derivative of the potential with respect to phi
@@ -732,15 +780,9 @@ double SCFPotentialPlanarphi2deriv(double R, double phi,
                                    double t,
                                    struct potentialArg * potentialArgs)
 {
-
-
     double Farray[3];
-
-    computeDeriv(R, 0, phi, t,potentialArgs, &Farray) ;
-
+    computeDeriv(R, 0, phi, t,potentialArgs, &Farray[0]) ;
     return *(Farray + 1);
-
-
 }
 
 //Compute the planar double derivative of the potential with respect to R, Phi
@@ -748,9 +790,7 @@ double SCFPotentialPlanarRphideriv(double R, double phi,
                                    double t,
                                    struct potentialArg * potentialArgs)
 {
-
     double Farray[3];
-    computeDeriv(R, 0, phi, t,potentialArgs, &Farray) ;
+    computeDeriv(R, 0, phi, t,potentialArgs, &Farray[0]) ;
     return *(Farray + 2);
-
 }
