@@ -3,6 +3,9 @@
  * at http://bigwww.epfl.ch/thevenaz/interpolation/
  ****************************************************************************/ 
  
+#ifdef __cplusplus
+extern "C" {
+#endif
 #include	<float.h>
 #include	<math.h>
 #include	<stddef.h>
@@ -12,3 +15,6 @@
 /*--------------------------------------------------------------------------*/
 void put_row(double *,long,double *,long);
 int samples_to_coefficients(double *,long,long);
+#ifdef __cplusplus
+}
+#endif
